<?php
/*
 * @copyright 2006 M3 Media Services
 * @version $Id:Field.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 * @since 0.3.19 - Feb 10, 2006
 *
 */

require_once MAX_PATH . '/lib/max/Dal/Reporting.php';
require_once MAX_PATH . '/lib/max/DateRange.php';
require_once MAX_PATH . '/lib/max/Admin/Reporting/ZoneScope.php';
require_once MAX_PATH . '/lib/max/Admin/Reporting/Field.php';

class DateRangeField extends Field
{
    /* @var bool */
    var $_week;
    /* @var Date */
    var $_now;
    /* @var DateRange */
    var $_range;

    function __construct($now = null)
    {
        $this->_week = false;
        $this->_now = $now;
        $this->_range = new DateRange($now);
    }

    function DateRangeField($now = null)
    {
        $this->__construct($now);
    }

    function isLastMonth()
    {
        if ($this->_week)
        {
            return false;
        }
        return true;
    }

    function isLastWeek()
    {
        return $this->_week;
    }

    function useLastWeek()
    {
        $this->_range->useLastWeek();
        $this->_week = true;
    }

    function getStartDate()
    {
        return $this->_range->getStartDate();
    }

    function getEndDate()
    {
        return $this->_range->getEndDate();
    }

    function _displaySelector($fields, $key)
    {
        $overall_name = $key;
        $preset_id = $overall_name . '_preset';
?>
        <select name='<?php print $preset_id; ?>' id='<?php print $preset_id; ?>' onchange='<?php print $key; ?>FormChange(this, 0, 1)' tabindex='<?php print $this->tabkeeper->tabindex++; ?>'>
            <option value='lastmonth' selected='selected'>Last month</option>
            <option value='lastweek'>Last week</option>
            <option value='last7days'>Last 7 days</option>
            <option value='thismonth'>This month</option>
            <option value='thisweek'>This week</option>
            <option value='specific'>Specific dates</option>
        </select>
<?php
    }

    function displayField($fields, $key)
    {
        $this->_displayFieldHeader($fields[$key]['title']);
        $overall_name = $key;
        $preset_id = $overall_name . '_preset';
        $start_id = $overall_name . '_start';
        $end_id = $overall_name . '_end';
        $start_button_id = $start_id . '_button';
        $end_button_id = $end_id . '_button';

        if (isset($fields[$key]['default'])) {
            $range = $this->getPassableValueFromQueryArray($fields[$key]['default'], $key);
        } else {
            $range = $this->getPassableValueFromQueryArray(array($key.'_preset' => 'thismonth'), $key);
        }

        $start_date = $range->getStartDate();
        $end_date = $range->getHumanEndDate();

        if (is_object($end_date) && $end_date->after(new Date())) {
            $end_date = new Date();
        }

        $this->_displaySelector($fields, $key);
?>
        <label for='<?php print $start_id; ?>'>From</label>
        <input class='date' name='<?php print $start_id; ?>' id='<?php print $start_id; ?>' type='text' value='<?php print is_null($start_date) ? '' : $start_date->format('%Y-%m-%d'); ?>'/>
        <button id='<?php print $start_button_id; ?>'>...</button>

        <label for='<?php print $end_id; ?>'>To</label>
        <input class='date' name='<?php print $end_id; ?>' id='<?php print $end_id; ?>' type='text' value='<?php print is_null($start_date) ? '' : $end_date->format('%Y-%m-%d'); ?>' />
        <button id='<?php print $end_button_id; ?>'>...</button>
<?php
        $this->_displayFieldFooter();
    }

    function displayExtrasAfterForm($fields, $key)
    {
        $overall_name = $key;
        $preset_id = $overall_name . '_preset';
        $start_id = $overall_name . '_start';
        $end_id = $overall_name . '_end';
        $start_button_id = $start_id . '_button';
        $end_button_id = $end_id . '_button';
        ?>
    <script type='text/javascript'>

        Calendar.setup({
            inputField : '<?php print $start_id; ?>',
            ifFormat   : '%Y-%m-%d',
            button     : '<?php print $start_button_id; ?>',
            align      : 'Bl',
            weekNumbers: false,
            firstDay   : <?php print $GLOBALS['pref']['begin_of_week'] ? 1 : 0; ?>,
            electric   : false
        })
        Calendar.setup({
            inputField : '<?php print $end_id; ?>',
            ifFormat   : '%Y-%m-%d',
            button     : '<?php print $end_button_id; ?>',
            align      : 'Bl',
            weekNumbers: false,
            firstDay   : <?php print $GLOBALS['pref']['begin_of_week'] ? 1 : 0; ?>,
            electric   : false
        })

        function <?php print $key; ?>FormChange(o, bReload, bReset) {
            var specific = o.options[o.selectedIndex].value == 'specific';

            document.getElementById('<?php print $start_id; ?>').disabled = !specific;
            document.getElementById('<?php print $start_button_id; ?>').disabled = !specific;
            document.getElementById('<?php print $end_id; ?>').disabled = !specific;
            document.getElementById('<?php print $end_button_id; ?>').disabled = !specific;

            if (bReset && !specific) {
                document.getElementById('<?php print $start_id; ?>').value =
                    document.getElementById('<?php print $end_id; ?>').value = '';
            }

            if (bReload && !specific)
                o.form.submit();
        }
        function <?php print $key; ?>FormSubmit() {
            document.getElementById('<?php print $preset_id; ?>').form.submit();
            return false;
        }
        <?php print $key; ?>FormChange(document.getElementById('<?php print $preset_id; ?>'), 0, 0);
    </script>

        <?php
    }

    function getPassableValueFromQueryArray($values, $base_key)
    {
        $range = new DateRange($this->_now);
        $range->useValuesFromQueryArray($values, $base_key);
        return $range;
    }

    function getRepresentativeValue()
    {
        return $this->_range;
    }
}
?>