<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id:IndexModule.php 4204 2006-02-10 09:55:36Z roh@m3.net $
*/

require_once MAX_PATH . '/lib/max/language/Report.php';
require_once MAX_PATH . '/www/admin/config.php';
require_once MAX_PATH . '/www/admin/lib-statistics.inc.php';
require_once MAX_PATH . '/lib/max/Plugin.php';
require_once 'FieldFactory.php';

class ReportIndexModule
{
    /* @var FieldFactory */
    var $field_factory;

    function ReportIndexModule()
    {
        $this->field_factory = new FieldFactory();
    }

    /**
     * Handle the report listing and the display of individual report parameters
     *
     * Previously in-line procedural code.
     * @todo Split responsibility of listing and of individual reports
     */
    function main($selection)
    {
        $this->tabindex = 1;

        $plugins = &MAX_Plugin::getPlugins('reports', null, false);
        // authorization
        foreach ($plugins as $pluginType => $plugin) {
            if (!$plugin->isAllowedToDisplay()) {
                continue;
            }
            $plugins[$pluginType]->info = $plugin->info();
        }

        // group plugins into plugin-category groups
        foreach ($plugins as $pluginKey => $plugin) {
            $groupedPlugins[$plugin->info['plugin-category-name']][$pluginKey] = $plugin;
        }

        if (!empty($plugins) && empty($selection)) {
            $this->displayPluginList($groupedPlugins);
        }
        else
        {
            $plugin = $plugins[$selection];
            $this->displayReportPluginInformation($plugin, $selection);
        }
    }

    /**
     * Displays full information about a report plugin.
     *
     * Includes title, description, and all parameters required to
     * execute it.
     *
     */
    function displayReportPluginInformation($plugin, $selection)
    {
        $this->_displayReportIntroduction($plugin->info);

        if ($fields = $plugin->info['plugin-import'])
        {
            $this->_displayParameterListHeader();

            foreach (array_keys($fields) as $key)
            {
                $field_type = $fields[$key]['type'];
                $field =& $this->field_factory->newField($field_type);

                $field->tabindex = $this->tabindex;
                $field->displayField($fields, $key);
                $this->tabindex = $field->tabindex;
            }
            $this->_displayParameterListFooter($selection, $fields);
        }
        $this->_displayReportInformationFooter();
    }

    function _displayParameterListHeader()
    {
        echo "<tr><td width='30'><img src='images/spacer.gif' height='1' width='100%'></td>";
        echo "<td colspan='2'><img src='images/break-l.gif' height='1' width='200' vspace='6'></td></tr>";

        echo "<form action='report-execute.php' method='get'>";
    }

    function _displayParameterListFooter($selection, $fields)
    {
        echo "<tr><td height='25' colspan='3'>";
        echo "<br><br>";
        echo "<input type='hidden' name='plugin' value='".$selection."'>";
        echo "<input type='submit' value='".$GLOBALS['strGenerate']."' tabindex='".($this->tabindex++)."'>";
        echo "&nbsp;";
        echo "<input type='button' value='".$GLOBALS['strBackToTheList']."' onClick='javascript:document.location.href=\"report-index.php\"' tabindex='".($this->tabindex++)."'>";
        echo "</td></tr>";

        echo "</form>";

        foreach(array_keys($fields) as $key) {
            $field_info = $fields[$key];
            $field_type = $field_info['type'];
            $field = $this->field_factory->newField($field_type);
            $field->displayExtrasAfterForm($field, $key);
        }
    }

    function _displayReportIntroduction($info)
    {
        echo "<table border='0' width='100%' cellpadding='0' cellspacing='0'>";
        echo "<tr><td height='25' colspan='3'>";

        if ($info['plugin-export'] == 'csv')
            echo "<img src='images/excel.gif' align='absmiddle'>&nbsp;";

        echo "<b>".$info['plugin-name']."</b></td></tr>";
        echo "<tr height='1'><td colspan='3' bgcolor='#888888'><img src='images/break.gif' height='1' width='100%'></td></tr>";

        echo "<tr><td height='10' colspan='3'>&nbsp;</td></tr>";
        echo "<tr><td width='30'>&nbsp;</td>";
        echo "<td height='25' colspan='2'>";
        echo nl2br($info['plugin-description']);
        echo "</td></tr>";
        echo "<tr><td height='10' colspan='3'>&nbsp;</td></tr>";
    }

    function _displayReportInformationFooter()
    {
        echo "</table>";
    }

    /**
     * Displays an HTML list of reports available to the logged-in user.
     *
     * Reports show as a title with a description,
     * and are grouped into categories.
     */
    function displayPluginList($groupedPlugins)
    {
            foreach ($groupedPlugins as $pluginCategoryName => $pluginsInGroup) {
                $this->_printCategoryTitle($pluginCategoryName);

                foreach ($pluginsInGroup as $key => $plugin) {
                    $this->_printPluginSummary($plugin, $key);
                }
                echo "
            <tr><td colspan='3'>&nbsp;</td></tr>";
            }
    }

    function _printCategoryTitle($pluginCategoryName)
    {
        echo "
            <table border='0' width='100%' cellpadding='0' cellspacing='0'>
            <tr>
                <td height='25' colspan='3'><b>{$pluginCategoryName}</b></td>
            </tr>
            <tr height='1'>
                <td width='30'><img src='images/break.gif' height='1' width='30'></td>
                <td width='200'><img src='images/break.gif' height='1' width='200'></td>
                <td width='100%'><img src='images/break.gif' height='1' width='100%'></td>
            </tr>
            ";
    }

    function _printPluginSummary($plugin, $key)
    {
        echo "
            <tr><td height='10' colspan='3'>&nbsp;</td></tr>
            <tr>
                <td width='30'>&nbsp;</td>
                <td width='200'><a href='report-index.php?selection=$key'>{$plugin->info['plugin-name']}</a></td>
                <td width='100%'>{$plugin->info['plugin-description']}</td>
            </tr>";
    }
}

?>