<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: mysql.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

/**
 * The mysql data access layer code the delivery engine.
 *
 * @package    MaxDal
 * @subpackage Delivery
 * @author     Chris Nutting <chris@m3.net>
 * @author     Andrew Hill <andrew@m3.net>
 */

/**
 * The function to connect to the database
 *
 * @return boolean  Returns true on sucessful creation of the database link
 *                  or false on failure
 */
function MAX_Dal_Delivery_connect() {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    // If a connection already exists, then return that
    if (isset($GLOBALS['_MAX']['ADMIN_DB_LINK']) && is_resource($GLOBALS['_MAX']['ADMIN_DB_LINK'])) {
        return $GLOBALS['_MAX']['ADMIN_DB_LINK'];
    } else {
        // Connect to the admin database
	    $dbPort     = isset($conf['database']['port']) ? $conf['database']['port'] : 3306;
	    $dbHost     = $dbPort != 3306 ? $conf['database']['host'].':'.$dbPort : $conf['database']['host'];
	    $dbUser     = $conf['database']['username'];
	    $dbPassword = $conf['database']['password'];
	    $dbName     = $conf['database']['name'];
	    if ($conf['database']['persistent']) {
	        $dbLink = @mysql_pconnect($dbHost, $dbUser, $dbPassword);
	    } else {
	        $dbLink = @mysql_connect($dbHost, $dbUser, $dbPassword);
	    }
	    if (@mysql_select_db($dbName, $dbLink)) {
	        return $dbLink;
	    }
    }
    return false;
}

/**
 * The function to pass a query to the database
 *
 * @param  string   $query      The SQL query to send to the database
 *
 * @return resource The handle to the MySQL resource for this query
 *               or false on failure
 */
function MAX_Dal_Delivery_query($query) {
    // Connect to the database if necessary
    if (empty($GLOBALS['_MAX']['ADMIN_DB_LINK'])) {
        $GLOBALS['_MAX']['ADMIN_DB_LINK'] = MAX_Dal_Delivery_connect();
    }
    if (is_resource($GLOBALS['_MAX']['ADMIN_DB_LINK'])) {
        return @mysql_query($query, $GLOBALS['_MAX']['ADMIN_DB_LINK']);
    } else {
        return false;
    }
}

/**
 * The function to get and return fields for a zone
 *
 * @param  int   $zoneid     The id of the zone to get linked ads for
 *
 * @return mixed The array containg zone information
 *               or false on failure
 */
function MAX_Dal_Delivery_getZoneInfo($zoneid) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    $rZoneInfo = MAX_Dal_Delivery_query("
    SELECT
        z.zoneid AS zone_id,
        z.affiliateid AS publisher_id,
        z.zonename AS name,
        z.delivery AS type,
        z.description AS description,
        z.width AS width,
        z.height AS height,
        z.chain AS chain,
        z.prepend AS prepend,
        z.append AS append,
        z.appendtype AS appendtype,
        z.forceappend AS forceappend,
        z.inventory_forecast_type AS inventory_forecast_type,
        p.default_banner_url AS default_banner_url,
        p.default_banner_destination AS default_banner_dest
    FROM
        {$conf['table']['prefix']}{$conf['table']['zones']} AS z,
        {$conf['table']['prefix']}{$conf['table']['affiliates']} AS a,
        {$conf['table']['prefix']}{$conf['table']['preference']} AS p
    WHERE
        z.zoneid={$zoneid}
      AND
        z.affiliateid = a.affiliateid
      AND
        p.agencyid = a.agencyid
    ");
    
    if (!is_resource($rZoneInfo)) {
        return false;
    }
    $aZoneInfo = mysql_fetch_assoc($rZoneInfo);
    
    if (empty($aZoneInfo['default_banner_url'])) {
        // Agency has no default banner, so overwrite with admin's
        $rAdminDefault = MAX_Dal_Delivery_query("
        SELECT
            p.default_banner_url AS default_banner_url,
            p.default_banner_destination AS default_banner_dest
        FROM
            {$conf['table']['prefix']}{$conf['table']['preference']} AS p
        WHERE
            p.agencyid = 0
        ");
        $aAdminDefaultBanner = mysql_fetch_assoc($rAdminDefault);
        $aZoneInfo['default_banner_url']  = $aAdminDefaultBanner['default_banner_url'];
        $aZoneInfo['default_banner_dest'] = $aAdminDefaultBanner['default_banner_dest'];
    }
    return ($aZoneInfo);
}
/**
 * The function to get and return the ads linked to a zone
 *
 * @param  int   $zoneid The id of the zone to get linked ads for
 * @return mixed The array containg zone information with nested arrays of linked ads
 *               or false on failure. Note that:
 *                  - Exclusive ads are in "xAds"
 *                  - Normal (paid) ads are in "ads"
 *                  - Low-priority ads are in "lAds"
 *                  - Companion ads, in addition to being in one of the above, are
 *                    also in "cAds" and "clAds"
 *                  - Exclusive and low-priority ads have had their priorities
 *                    calculated on the basis of the placement and advertisement
 *                    weight
 */
function MAX_Dal_Delivery_getZoneLinkedAds($zoneid) {
    $conf = $GLOBALS['_MAX']['CONF'];
    $aRows = MAX_Dal_Delivery_getZoneInfo($zoneid);
    
    $aRows['xAds']  = array();
    $aRows['cAds']  = array();
    $aRows['clAds'] = array();
    $aRows['ads']   = array();
    $aRows['lAds']  = array();
    $aRows['count_active'] = 0;
    $aRows['zone_companion'] = false;
    $aRows['count_active'] = 0;
    
    $totals = array(
        'xAds'  => 0,
        'cAds'  => 0,
        'clAds' => 0,
        'ads'   => 0,
        'lAds'  => 0
    );
    
    $rAds = MAX_Dal_Delivery_query("
        SELECT
            d.bannerid AS ad_id,
            d.campaignid AS placement_id,
            d.active AS active,
            d.description AS name,
            d.storagetype AS type,
            d.contenttype AS contenttype,
            d.pluginversion AS pluginversion,
            d.filename AS filename,
            d.imageurl AS imageurl,
            d.htmltemplate AS htmltemplate,
            d.htmlcache AS htmlcache,
            d.width AS width,
            d.height AS height,
            d.weight AS weight,
            d.seq AS seq,
            d.target AS target,
            d.url AS url,
            d.alt AS alt,
            d.status AS status,
            d.bannertext AS bannertext,
            d.autohtml AS autohtml,
            d.adserver AS adserver,
            d.block AS block,
            d.capping AS capping,
            d.session_capping AS session_capping,
            d.compiledlimitation AS compiledlimitation,
            d.acl_plugins AS acl_plugins,
            d.append AS append,
            d.appendtype AS appendtype,
            d.bannertype AS bannertype,
            d.alt_filename AS alt_filename,
            d.alt_imageurl AS alt_imageurl,
            d.alt_contenttype AS alt_contenttype,
            c.priority AS campaign_priority,
            c.weight AS campaign_weight,
            c.companion AS campaign_companion,
            az.priority AS priority
        FROM
            {$conf['table']['prefix']}{$conf['table']['banners']} AS d,
            {$conf['table']['prefix']}{$conf['table']['ad_zone_assoc']} AS az,
            {$conf['table']['prefix']}{$conf['table']['campaigns']} AS c
        WHERE
            az.zone_id = {$zoneid}
          AND
            d.bannerid = az.ad_id
          AND
            c.campaignid = d.campaignid
          AND
            d.active = 't'
          AND
            c.active = 't'
    ");
    
    if (!is_resource($rAds)) {
        return false;
    }
    while ($aAd = mysql_fetch_assoc($rAds)) {
        // Is the ad Exclusive, Low, or Normal Priority?
        if ($aAd['campaign_priority'] == -1) {
            // Ad is in an exclusive placement
            $aAd['priority'] = $aAd['campaign_weight'] * $aAd['weight'];
            $aRows['xAds'][$aAd['ad_id']] = $aAd;
            $aRows['count_active']++;
            $totals['xAds'] += $aAd['priority'];
        } elseif ($aAd['campaign_priority'] == 0) {
            // Ad is in a low priority placement
            $aAd['priority'] = $aAd['campaign_weight'] * $aAd['weight'];
            $aRows['lAds'][$aAd['ad_id']] = $aAd;
            $aRows['count_active']++;
            $totals['lAds'] += $aAd['priority'];
        } else {
            // Ad is in a paid placement
            $aRows['ads'][$aAd['ad_id']] = $aAd;
            $aRows['count_active']++;
        }
        // Also store Companion ads in additional array
        if ($aAd['campaign_companion'] == 1) {
            if ($aAd['campaign_priority'] == 0) {
                // Store a low priority companion ad
                $aRows['zone_companion'][] = $aAd['placement_id'];
                $aRows['clAds'][$aAd['ad_id']] = $aAd;
                $totals['clAds'] += $aAd['priority'];    
            } else {
                // Sore a paid priority companion ad
                $aRows['zone_companion'][] = $aAd['placement_id'];
                $aRows['cAds'][$aAd['ad_id']] = $aAd;
            }
            
        }
    }
    // If there are paid ads, sort by priority, and set the total to 1,
    // to ensure blank ads are selected (when required)
    if (is_array($aRows['ads'])) {
        uasort($aRows['ads'], 'MAX_Dal_Delivery_sortArrayPriority');
        $totals['ads'] = 1;
    }
    // If there are low priority ads, sort by priority
    if (is_array($aRows['lAds'])) {
        uasort($aRows['lAds'], 'MAX_Dal_Delivery_sortArrayPriority');
    }
    // If there are paid companion ads, sort by priority, and set the
    // total to 1, to ensure blank ads are selected (when required)
    if (is_array($aRows['cAds'])) {
        uasort($aRows['cAds'], 'MAX_Dal_Delivery_sortArrayPriority');
        $totals['cAds'] = 1;
    }
    // If there are low priority companion ads, sort by priority
    if (is_array($aRows['clAds'])) {
        uasort($aRows['clAds'], 'MAX_Dal_Delivery_sortArrayPriority');
    }
    $aRows['priority'] = $totals;
    return $aRows;
}

/**
 * The function to get and return the ads for direct selection
 *
 * @param  string   $search     The search string for this banner selection
 *                              Usually 'bannerid:123' or 'campaignid:123'
 *
 * @return mixed The array of ads matching the search criteria
 *               or false on failure
 */

function MAX_Dal_Delivery_getLinkedAds($search) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    // Deal with categories
    $where1 = preg_replace('/cat:(\w+)/', "cat.name='$1'", $search);
    // Deal with sizes
    $where2 = preg_replace('/size:(\d+)x(\d+)/', 'd.width=$1 AND d.height=$2', $where1);
    // Deal with ads
    $where3 = preg_replace('/(ad_id|adid|bannerid):(\d+)/', 'd.bannerid=$2', $where2);
    // Deal with campaigns
    $where4 = preg_replace('/(placement_id|placementid|campaignid):(\d+)/', 'd.campaignid=$2', $where3);
    // Deal with width, height
    $where = preg_replace('/(width|height):(\d+)/', 'd.$1=$2', $where4);
    
    $aColumns = array(
        'd.bannerid AS ad_id',
        'd.campaignid AS placement_id',
        'd.active AS active',
        'd.description AS name',
        'd.storagetype AS type',
        'd.contenttype AS contenttype',
        'd.pluginversion AS pluginversion',
        'd.filename AS filename',
        'd.imageurl AS imageurl',
        'd.htmltemplate AS htmltemplate',
        'd.htmlcache AS htmlcache',
        'd.width AS width',
        'd.height AS height',
        'd.weight AS weight',
        'd.seq AS seq',
        'd.target AS target',
        'd.url AS url',
        'd.alt AS alt',
        'd.status AS status',
        'd.bannertext AS bannertext',
        'd.autohtml AS autohtml',
        'd.adserver AS adserver',
        'd.block AS block',
        'd.capping AS capping',
        'd.session_capping AS session_capping',
        'd.compiledlimitation AS compiledlimitation',
        'd.append AS append',
        'd.appendtype AS appendtype',
        'd.bannertype AS bannertype',
        'd.alt_filename AS alt_filename',
        'd.alt_imageurl AS alt_imageurl',
        'd.alt_contenttype AS alt_contenttype',
        'az.priority AS priority'
    );
    $aTables = array(
        $conf['table']['prefix'].$conf['table']['banners'] . ' AS d',
        $conf['table']['prefix'].$conf['table']['campaigns'] . ' AS m',
        $conf['table']['prefix'].$conf['table']['ad_zone_assoc'] . ' AS az'
    );
    
    if ($where1 != $search) {
        $aTables[] = $conf['table']['prefix'].$conf['table']['ad_category_assoc'] . ' AS ac';
        $aTables[] = $conf['table']['prefix'].$conf['table']['category'] . ' AS cat';
        $where = 'd.bannerid=ac.ad_id AND ac.category_id=cat.category_id AND ' . $where;
    }
    
    $columns = implode(",\n    ", $aColumns);
    $tables = implode(",\n    ", $aTables);    
    $where = "
    d.bannerid=az.ad_id
  AND az.zone_id=0
  AND d.campaignid=m.campaignid
  AND m.active='t'
  AND d.active='t'
  AND {$where}";
    
    $query = "SELECT\n    " . $columns . "\nFROM\n    " . $tables . "\nWHERE " . $where;
    
    $rAds = MAX_Dal_Delivery_query($query);
    
    if (!is_resource($rAds)) {
        return false;
    }
    while ($aRow = mysql_fetch_assoc($rAds)) {
            $aRows[] = $aRow;
    }
    if (is_array($aRows)) {
        uasort($aRows, 'MAX_Dal_Delivery_sortArrayPriority');
    }
    $aAds = array();
    $aAds['ads'] = $aRows;
    $aAds['priority']['ads'] = 0;
    if (is_array($aRows)) {
        foreach ($aRows as $aRow) {
            $aAds['priority']['ads'] += $aRow['priority'];
        }
    }
    return $aAds;
}

/**
 * The function to get and return a single ad (for popup page code generation)
 *
 * @param  string   $ad_id     The ad id for the specified ad
 *
 * @return mixed    $ad        An array containing the ad data
 */
function MAX_Dal_Delivery_getAd($ad_id) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    $rAd = MAX_Dal_Delivery_query("
        SELECT
        d.bannerid AS ad_id,
        d.campaignid AS placement_id,
        d.active AS active,
        d.description AS name,
        d.storagetype AS type,
        d.contenttype AS contenttype,
        d.pluginversion AS pluginversion,
        d.filename AS filename,
        d.imageurl AS imageurl,
        d.htmltemplate AS htmltemplate,
        d.htmlcache AS htmlcache,
        d.width AS width,
        d.height AS height,
        d.weight AS weight,
        d.seq AS seq,
        d.target AS target,
        d.url AS url,
        d.alt AS alt,
        d.status AS status,
        d.bannertext AS bannertext,
        d.autohtml AS autohtml,
        d.adserver AS adserver,
        d.block AS block,
        d.capping AS capping,
        d.session_capping AS session_capping,
        d.compiledlimitation AS compiledlimitation,
        d.append AS append,
        d.appendtype AS appendtype,
        d.bannertype AS bannertype,
        d.alt_filename AS alt_filename,
        d.alt_imageurl AS alt_imageurl,
        d.alt_contenttype AS alt_contenttype
    FROM
        {$conf['table']['prefix']}{$conf['table']['banners']} AS d
    WHERE
        d.bannerid={$ad_id}
    ");
    if (!is_resource($rAd)) {
        return false;
    } else {
        return (mysql_fetch_assoc($rAd));
    }
}

/**
 * The function to get and return a limitation
 *
 * @param  string   $channelid    The ad id for the specified ad
 *
 * @return string    $limitations  A string containing limitations for channel
 */
function MAX_Dal_Delivery_getChannelLimiations($channelid) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    $rLimitation = MAX_Dal_Delivery_query("
    SELECT
            acl_plugins,compiledlimitation
    FROM
            {$conf['table']['prefix']}{$conf['table']['channel']}
    WHERE
            channelid={$channelid}");
    if (!is_resource($rLimitation)) {
        return false;
    }
    $limitations = mysql_fetch_assoc($rLimitation);
    return $limitations;
}

/**
 * A private callback function for the uasort function
 *
 * @param  string   $a      First parameter
 *
 * @param  string   $b      Second parameter
 *
 * @return boolean  Compare resule
 */
function MAX_Dal_Delivery_sortArrayPriority($a, $b)
{
    $compare = ($a['priority'] > $b['priority']) ? -1 : 1;
    return $compare;
}

?>
