<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: mysql.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once MAX_PATH . '/lib/max/Dal/Statistics/Common.php';
require_once MAX_PATH . '/lib/max/other/lib-userlog.inc.php';
require_once 'Date.php';

/**
 * The data access layer code for summarising raw data into statistics, for
 * the AdServer module.
 *
 * @package    MaxDal
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@m3.net>
 */
class MAX_Dal_Statistics_AdServer_mysql extends MAX_Dal_Statistics_Common
{
    var $dbh;
    var $sortBufferSize;

    /**
     * The constructor method.
     *
     * @uses MAX_Dal_Statistics_Common::MAX_Dal_Statistics_Common()
     */
    function MAX_Dal_Statistics_AdServer_mysql()
    {
        parent::MAX_Dal_Statistics_Common('mysql');
        $this->dbh = &MAX_DB::singleton();
        // Store the original MySQL sort_buffer_size value
        $query = "SHOW SESSION VARIABLES like 'sort_buffer_size'";
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $row = $result->fetchRow();
        if (is_array($row) && (count($row) == 2) && (isset($row['Value']))) {
            $this->sortBufferSize = $row['Value'];
        }
    }

    /**
     * A method to set the MySQL sort_buffer_size session variable,
     * if appropriate.
     *
     * @access private
     */
    function setSortBufferSize()
    {
        // Only set if the original sort_buffer_size is stored, and
        // if a specified value for the sort_buffer_size has been
        // defined by the user in the configuration
        $conf = $GLOBALS['_MAX']['CONF'];
        if (isset($this->sortBufferSize) && isset($conf['databaseMysql']['statisticsSortBufferSize']) &&
            is_numeric($conf['databaseMysql']['statisticsSortBufferSize'])) {
            $query = 'SET SESSION sort_buffer_size='.$conf['databaseMysql']['statisticsSortBufferSize'];
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
        }
    }

    /**
     * A method to restore the MySQL sort_buffer_size session variable,
     * if appropriate.
     *
     * @access private
     */
    function restoreSortBufferSize()
    {
        // Only restore if the original sort_buffer_size is stored,
        // and if a specified value for the sort_buffer_size has
        // been defined by the user in the configuration
        $conf = $GLOBALS['_MAX']['CONF'];
        if (isset($this->sortBufferSize) && isset($conf['databaseMysql']['statisticsSortBufferSize']) &&
            is_numeric($conf['databaseMysql']['statisticsSortBufferSize'])) {
            $query = 'SET SESSION sort_buffer_size='.$this->sortBufferSize;
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
        }
    }

    /**
     * A method to find the last time that maintenance statistics was run.
     *
     * @param integer $type The update type that occurred - that is,
     *                      DAL_STATISTICS_COMMON_UPDATE_OI if the update was
     *                      done on the basis of the operation interval,
     *                      DAL_STATISTICS_COMMON_UPDATE_HOUR if the update
     *                      was done on the basis of the hour, or
     *                      DAL_STATISTICS_COMMON_UPDATE_BOTH if the update
     *                      was done on the basis of both the operation
     *                      interval and the hour.
     * @param Date $now An optional Date, used to specify the "current time", and
     *                  to limit the method to only look for past maintenance
     *                  statistics runs before this date. Normally only used
     *                  to assist with re-generation of statistics in the event
     *                  of faulty raw tables.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function getMaintenanceStatisticsLastRunInfo($type, $now = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_raw_ad_impression'];
        return $this->_getMaintenanceStatisticsLastRunInfo($type, $table, $now);
    }

    /**
     * A private function to do the job of
     * {@link MAX_Dal_Statistics_AdServer::getMaintenanceStatisticsLastRunInfo()}, but with an
     * extra parameter to specify the raw table to look in, in the case of
     * maintenance statistics not having been run before.
     *
     * @access private
     * @param integer $type The update type that occurred - that is,
     *                      DAL_STATISTICS_COMMON_UPDATE_OI if the update was
     *                      done on the basis of the operation interval,
     *                      DAL_STATISTICS_COMMON_UPDATE_HOUR if the update
     *                      was done on the basis of the hour, or
     *                      DAL_STATISTICS_COMMON_UPDATE_BOTH if the update
     *                      was done on the basis of both the operation
     *                      interval and the hour.
     * @param string $rawTable The raw table to use in case of no previous run.
     * @param Date $now An optional Date, used to specify the "current time", and
     *                  to limit the method to only look for past maintenance
     *                  statistics runs before this date. Normally only used
     *                  to assist with re-generation of statistics in the event
     *                  of faulty raw tables.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function _getMaintenanceStatisticsLastRunInfo($type, $rawTable, $now = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($type == DAL_STATISTICS_COMMON_UPDATE_OI) {
            $whereClause = 'WHERE (lms.adserver_run_type = ' . DAL_STATISTICS_COMMON_UPDATE_OI .
                           ' OR lms.adserver_run_type = ' . DAL_STATISTICS_COMMON_UPDATE_BOTH. ')';
        } elseif ($type == DAL_STATISTICS_COMMON_UPDATE_HOUR) {
            $whereClause = 'WHERE (lms.adserver_run_type = ' . DAL_STATISTICS_COMMON_UPDATE_HOUR .
                           ' OR lms.adserver_run_type = ' . DAL_STATISTICS_COMMON_UPDATE_BOTH . ')';
        } elseif ($type == DAL_STATISTICS_COMMON_UPDATE_BOTH) {
            $whereClause = 'WHERE (lms.adserver_run_type = ' . DAL_STATISTICS_COMMON_UPDATE_BOTH . ')';
        } else {
            MAX::debug('Invalid update type value ' . $type, PEAR_LOG_ERR);
            MAX::debug('Aborting script execution', PEAR_LOG_ERR);
            exit();
        }
        if (!is_null($now)) {
            // Limit to past maintenance statistics runs before this Date
            $whereClause .= ' AND updated_to < ' . "'" . $now->format('%Y-%m-%d %H:%M:%S') . "'";
        }
        $query = "
            SELECT
                lms.updated_to AS date
            FROM
                {$conf['table']['prefix']}{$conf['table']['log_maintenance_statistics']} AS lms
            $whereClause
            ORDER BY date DESC
            LIMIT 1";
        $message = 'Getting the details of when maintenance statistics last ran for the AdServer module ' .
                   'on the basis of the ';
        if ($type == DAL_STATISTICS_COMMON_UPDATE_OI) {
            $message .= 'operation interval';
        } elseif ($type == DAL_STATISTICS_COMMON_UPDATE_HOUR) {
            $message .= 'hour';
        } elseif ($type == DAL_STATISTICS_COMMON_UPDATE_BOTH) {
            $message .= 'both the opertaion interval and hour';
        }
        MAX::debug($message, PEAR_LOG_DEBUG);
        $row = $this->dbh->getRow($query);
        if (PEAR::isError($row)) {
            return MAX::raiseError($row, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        if ($row) {
            $date = &new Date($row['date']);
            return $date;
        } else {
            // Maintenance statistics has never been run - query the raw data
            $query = "
                SELECT
                    raw.date_time AS date
                FROM
                    $rawTable AS raw
                ORDER BY date ASC
                LIMIT 1";
            MAX::debug('Getting the details of when the first ad impression is for the AdServer module',
                       PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result) && !PEAR::isError($result, DB_ERROR_NOSUCHTABLE)) {
                MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            if ($result->numRows() > 0) {
                $row = $result->fetchRow();
                // Convert the impression time into the end of the previous operation interval or hour
                $date = &new Date($row['date']);
                if ($type == 0) {
                    list($date, $_) = $this->maintenance->convertDateToOperationIntervalStartAndEndDates($date);
                    $date->subtractSeconds(1);
                    return $date;
                } else {
                    $date = &new Date($date->format('%Y-%m-%d %H:00:00'));
                    $date->subtractSeconds(1);
                    return $date;
                }
            }
            // No raw data was found
            return null;
        }
    }

   /**
    * Method to summarise request, impression or click data.
    *
    * @param String $type type of data to summarise
    * @param Date $oStart start of interval to be summarised
    * @param Date $oEnd end of interval to be summarised
    * @return Integer affected rows
    */
    function _summariseData($type, $oStart, $oEnd)
    {
    	$conf = $GLOBALS['_MAX']['CONF'];
    	if (empty($type)) {
    		return 0;
    	}
    	$type = strtolower($type);
    	$tmpTableName = 'tmp_ad_' . $type;
    	$countColumnName = $type . 's';
        // Check the start and end dates
        if (!$this->maintenance->checkIntervalDates($oStart, $oEnd, $conf['maintenance']['operationInterval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            $this->maintenance->convertDateToOperationIntervalStartAndEndDates($oStart);
        // Get the operation interval ID
        $operationIntervalID = $this->maintenance->convertDateToOperationIntervalID($operationIntervalStart);
        // Construct temporary summary table name and create
        $this->tempTables->createTable($tmpTableName);
        // Set the MySQL sort buffer size
        $this->setSortBufferSize();
        // Summarise the requests
        $baseTable = $conf['table']['prefix'] . $conf['table']['data_raw_ad_' . $type];
        $query = "
            INSERT INTO
                $tmpTableName
                (
                    day,
                    hour,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    creative_id,
                    zone_id,
                    $countColumnName
                )
            SELECT
                DATE_FORMAT(drad.date_time, '%Y-%m-%d') AS day,
                DATE_FORMAT(drad.date_time, '%k') AS hour,
                {$conf['maintenance']['operationInterval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drad.ad_id AS ad_id,
                drad.creative_id AS creative_id,
                drad.zone_id AS zone_id,
                COUNT(*) AS $countColumnName
            FROM
                $baseTable AS drad
            WHERE
                drad.date_time >= ".$oStart->format('%Y%m%d%H%M%S')."
                AND drad.date_time <= ".$oEnd->format('%Y%m%d%H%M%S')."
            GROUP BY
                day, hour, ad_id, creative_id, zone_id";
        MAX::debug("Summarising ad $type" . "s from the $baseTable table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows = $this->dbh->affectedRows();
        // Restore the MySQL sort buffer size
        $this->restoreSortBufferSize();
        return $rows;
    }

    /**
     * A method for summarising requests into a temporary table.
     *
     * @param Date $oStart The start date/time to summarise from.
     * @param Date $oEnd The end date/time to summarise to.
     * @return integer The number of request rows summarised.
     */
    function summariseRequests($oStart, $oEnd)
    {
    	return $this->_summariseData('request', $oStart, $oEnd);
    }

    /**
     * A method for summarising impressions into a temporary table.
     *
     * @param Date $oStart The start date/time to summarise from.
     * @param Date $oEnd The end date/time to summarise to.
     * @return integer The number of impression rows summarised.
     */
    function summariseImpressions($oStart, $oEnd)
    {
    	return $this->_summariseData('impression', $oStart, $oEnd);
    }

    /**
     * A method for summarising clicks into a temporary table.
     *
     * @param Date $oStart The start date/time to summarise from.
     * @param Date $oEnd The end date/time to summarise to.
     * @return integer The number of click rows summarised.
     */
    function summariseClicks($oStart, $oEnd)
    {
    	return $this->_summariseData('click', $oStart, $oEnd);
    }

    /**
     * A method for summarising connections into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of connection rows summarised.
     */
    function summariseConnections($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (!$conf['modules']['Tracker']) {
            return 0;
        }
        // Check the start and end dates
        if (!$this->maintenance->checkIntervalDates($start, $end, $conf['maintenance']['operationInterval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            $this->maintenance->convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = $this->maintenance->convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary tables
        $this->tempTables->createTable('tmp_tracker_impression_ad_impression_connection');
        $this->tempTables->createTable('tmp_tracker_impression_ad_click_connection');
        $this->tempTables->createTable('tmp_ad_connection');
        // Insert tracker impressions that may connect to ad impressions into
        // the temporary tmp_tracker_impression_ad_impression_connection table
        $query = "
            INSERT INTO
                tmp_tracker_impression_ad_impression_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    tracker_id,
                    campaign_id,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    viewer_id,
                    date_time,
                    connection_type,
                    connection_window,
                    connection_status
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.tracker_id AS tracker_id,
                ct.campaignid AS campaign_id,
                {$conf['maintenance']['operationInterval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drti.viewer_id AS viewer_id,
                drti.date_time AS date_time,
                '" . MAX_CONNECTION_AD_IMPRESSION . "' AS connection_type,
                ct.viewwindow AS connection_window,
                ct.status AS connection_status
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']} AS drti,
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']} AS ct
            WHERE
                drti.tracker_id = ct.trackerid
                AND drti.viewer_id != ''
                AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                AND ct.viewwindow > 0";
        MAX::debug('Selecting tracker impressions that may connect to ad impression into the ' .
                   'tmp_tracker_impression_ad_impression_connection temporary table', PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        // Connect the tracker impressions with ad impressions, where possible
        $query = "
            INSERT INTO
                tmp_ad_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    date_time,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    connection_viewer_id,
                    connection_viewer_session_id,
                    connection_date_time,
                    connection_ad_id,
                    connection_creative_id,
                    connection_zone_id,
                    connection_channel,
                    connection_language,
                    connection_ip_address,
                    connection_host_name,
                    connection_country,
                    connection_https,
                    connection_domain,
                    connection_page,
                    connection_query,
                    connection_referer,
                    connection_search_term,
                    connection_user_agent,
                    connection_os,
                    connection_browser,
                    connection_type,
                    connection_window,
                    connection_status
                )
            SELECT DISTINCT
                ttiaic.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                ttiaic.server_raw_ip AS server_raw_ip,
                ttiaic.date_time AS date_time,
                ttiaic.operation_interval AS operation_interval,
                ttiaic.operation_interval_id AS operation_interval_id,
                ttiaic.interval_start AS interval_start,
                ttiaic.interval_end AS interval_end,
                drai.viewer_id AS connection_viewer_id,
                drai.viewer_session_id AS connection_viewer_session_id,
                drai.date_time AS connection_date_time,
                drai.ad_id AS connection_ad_id,
                drai.creative_id AS connection_creative_id,
                drai.zone_id AS connection_zone_id,
                drai.channel AS connection_channel,
                drai.language AS connection_language,
                drai.ip_address AS connection_ip_address,
                drai.host_name AS connection_host_name,
                drai.country AS connection_country,
                drai.https AS connection_https,
                drai.domain AS connection_domain,
                drai.page AS connection_page,
                drai.query AS connection_query,
                drai.referer AS connection_referer,
                drai.search_term AS connection_search_term,
                drai.user_agent AS connection_user_agent,
                drai.os AS connection_os,
                drai.browser AS connection_browser,
                ttiaic.connection_type AS connection_type,
                ttiaic.connection_window AS connection_window,
                ttiaic.connection_status AS connection_status
            FROM
                tmp_tracker_impression_ad_impression_connection AS ttiaic,
                {$conf['table']['prefix']}{$conf['table']['data_raw_ad_impression']} AS drai,
                {$conf['table']['prefix']}{$conf['table']['banners']} AS b,
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']} AS ct
            WHERE
                ttiaic.viewer_id = drai.viewer_id
                AND drai.ad_id = b.bannerid
                AND b.campaignid = ct.campaignid
                AND ct.trackerid = ttiaic.tracker_id
                AND ttiaic.campaign_id = ct.campaignid
                AND ttiaic.date_time < DATE_ADD(drai.date_time, INTERVAL ttiaic.connection_window SECOND)
                AND ttiaic.date_time >= drai.date_time";
        MAX::debug('Connecting tracker impressions with ad impressions, where possible', PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows += $this->dbh->affectedRows();
        // Drop the temporary table for storing tracker impressions that may connect to ad impressions
        $this->tempTables->dropTempTable('tmp_tracker_impression_ad_impression_connection');
        // Insert tracker impressions that may connect to ad clicks into
        // the temporary tmp_tracker_impression_ad_click_connection table
        $query = "
            INSERT INTO
                tmp_tracker_impression_ad_click_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    tracker_id,
                    campaign_id,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    viewer_id,
                    date_time,
                    connection_type,
                    connection_window,
                    connection_status
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.tracker_id AS tracker_id,
                ct.campaignid AS campaign_id,
                {$conf['maintenance']['operationInterval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drti.viewer_id AS viewer_id,
                drti.date_time AS date_time,
                '" . MAX_CONNECTION_AD_CLICK . "' AS connection_type,
                ct.clickwindow AS connection_window,
                ct.status AS connection_status
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']} AS drti,
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']} AS ct
            WHERE
                drti.tracker_id = ct.trackerid
                AND drti.viewer_id != ''
                AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                AND ct.clickwindow > 0";
        MAX::debug('Selecting tracker impressions that may connect to ad clicks into the ' .
                   'tmp_tracker_impression_ad_click_connection temporary table', PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        // Connect the tracker impressions with ad clicks, where possible
        $query = "
            INSERT INTO
                tmp_ad_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    date_time,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    connection_viewer_id,
                    connection_viewer_session_id,
                    connection_date_time,
                    connection_ad_id,
                    connection_creative_id,
                    connection_zone_id,
                    connection_channel,
                    connection_language,
                    connection_ip_address,
                    connection_host_name,
                    connection_country,
                    connection_https,
                    connection_domain,
                    connection_page,
                    connection_query,
                    connection_referer,
                    connection_search_term,
                    connection_user_agent,
                    connection_os,
                    connection_browser,
                    connection_type,
                    connection_window,
                    connection_status
                )
            SELECT DISTINCT
                ttiacc.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                ttiacc.server_raw_ip AS server_raw_ip,
                ttiacc.date_time AS date_time,
                ttiacc.operation_interval AS operation_interval,
                ttiacc.operation_interval_id AS operation_interval_id,
                ttiacc.interval_start AS interval_start,
                ttiacc.interval_end AS interval_end,
                drac.viewer_id AS connection_viewer_id,
                drac.viewer_session_id AS connection_viewer_session_id,
                drac.date_time AS connection_date_time,
                drac.ad_id AS connection_ad_id,
                drac.creative_id AS connection_creative_id,
                drac.zone_id AS connection_zone_id,
                drac.channel AS connection_channel,
                drac.language AS connection_language,
                drac.ip_address AS connection_ip_address,
                drac.host_name AS connection_host_name,
                drac.country AS connection_country,
                drac.https AS connection_https,
                drac.domain AS connection_domain,
                drac.page AS connection_page,
                drac.query AS connection_query,
                drac.referer AS connection_referer,
                drac.search_term AS connection_search_term,
                drac.user_agent AS connection_user_agent,
                drac.os AS connection_os,
                drac.browser AS connection_browser,
                ttiacc.connection_type AS connection_type,
                ttiacc.connection_window AS connection_window,
                ttiacc.connection_status AS connection_status
            FROM
                tmp_tracker_impression_ad_click_connection AS ttiacc,
                {$conf['table']['prefix']}{$conf['table']['data_raw_ad_click']} AS drac,
                {$conf['table']['prefix']}{$conf['table']['banners']} AS b,
                {$conf['table']['prefix']}{$conf['table']['campaigns_trackers']} AS ct
            WHERE
                ttiacc.viewer_id = drac.viewer_id
                AND drac.ad_id = b.bannerid
                AND b.campaignid = ct.campaignid
                AND ct.trackerid = ttiacc.tracker_id
                AND ttiacc.campaign_id = ct.campaignid
                AND ttiacc.date_time < DATE_ADD(drac.date_time, INTERVAL ttiacc.connection_window SECOND)
                AND ttiacc.date_time >= drac.date_time";
        MAX::debug('Connecting tracker impressions with ad clicks, where possible', PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows += $this->dbh->affectedRows();
        // Drop the temporary table for storing tracker impressions that may connect to ad clicks
        $this->tempTables->dropTempTable('tmp_tracker_impression_ad_click_connection');
        return $rows;
    }

    /**
     * A method to update the intermediate tables with summarised data.
     *
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function saveIntermediate($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        // Check the start and end dates
        if (!$this->maintenance->checkIntervalDates($start, $end, $conf['maintenance']['operationInterval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            $this->maintenance->convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = $this->maintenance->convertDateToOperationIntervalID($operationIntervalStart);
        // Save connections to the intermediate tables
        if ($conf['modules']['Tracker']) {
            // Select the tracker impression/ad impression and tracker impression/ad click connections
            // that the most recent connection, in the case of duplicate connections per tracker
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_connection_latest
                TYPE={$conf['table']['type']}
                SELECT
                    tac.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                    tac.server_raw_ip AS server_raw_ip,
                    MAX(tac.connection_date_time) AS connection_date_time
                FROM
                    tmp_ad_connection AS tac
                WHERE
                    tac.date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND tac.date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'
                GROUP BY
                    tac.server_raw_tracker_impression_id, tac.server_raw_ip";
            MAX::debug('Selecting the tracker impression/ad impression and/or tracker impression/ad click ' .
                       'connections that have the most recent connection (ie. ad impression and/or click) '.
                       'date_time field', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Mark these "latest" connections
            $query = "
                UPDATE
                    tmp_ad_connection AS tac,
                    tmp_connection_latest AS tcl
                SET
                    tac.latest = 1
                WHERE
                    tac.server_raw_tracker_impression_id = tcl.server_raw_tracker_impression_id
                    AND tac.server_raw_ip = tcl.server_raw_ip
                    AND tac.connection_date_time = tcl.connection_date_time";
            MAX::debug('Setting the \'latest connection\' flag in the temporary tmp_connection table',
                       PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            $connectionRows = $this->dbh->affectedRows();
            // Drop the tmp_connection_latest table
            $this->tempTables->dropTempTable('tmp_connection_latest');
            if ($connectionRows > 0) {
                // Save the "latest" connections and the associated variable values
                $this->_saveConnectionsAndVariableValues($start, $end);
            }
            // Drop the tmp_ad_connection table
            $this->tempTables->dropTempTable('tmp_ad_connection');
        }
        // Summarise requests, impressions, clicks, and conversions (if needed)
        // into the data_intermediate_ad table
        if ($conf['modules']['Tracker']) {
            $this->tempTables->createTable('tmp_conversions');
            // Select all conversions that have basket variable values
            // attached into the tmp_conversions table.
            $query = "
                INSERT INTO
                    tmp_conversions
                    (
                        data_intermediate_ad_connection_id,
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        basket_value
                    )
                SELECT
                    diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,
                    DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(diac.tracker_date_time, '%k') AS hour,
                    {$conf['maintenance']['operationInterval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    diac.ad_id AS ad_id,
                    diac.creative_id AS creative_id,
                    diac.zone_id AS zone_id,
                    SUM(diavv.value) AS basket_value
                FROM
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac
                LEFT JOIN
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS diavv
                USING
                    (
                        data_intermediate_ad_connection_id
                    )
                LEFT JOIN
                    {$conf['table']['prefix']}{$conf['table']['variables']} AS v
                ON
                    (
                        diavv.tracker_variable_id = v.variableid
                    )
                WHERE
                    diac.connection_status = ". MAX_CONNECTION_STATUS_APPROVED ."
                    AND v.is_basket_value = 1
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'
                GROUP BY
                    diac.data_intermediate_ad_connection_id, day, hour, ad_id, creative_id, zone_id";
            MAX::debug('Selecting conversions with basket values', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Get the ID of all conversions with basket variable values
            $query = "
                SELECT DISTINCT
                    data_intermediate_ad_connection_id
                FROM
                    tmp_conversions";
            MAX::debug('Getting all conversion IDs which have basket values', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            $aIds = array();
            while ($row = $result->fetchRow()) {
                $aIds[] = $row['data_intermediate_ad_connection_id'];
            }
            // Select all conversions that have non-basket variable values
            // attached, which also do NOT have basket variable values
            // attached, into the tmp_conversions table
            $query = "
                INSERT INTO
                    tmp_conversions
                    (
                        data_intermediate_ad_connection_id,
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        basket_value
                    )
                SELECT
                    diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,
                    DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(diac.tracker_date_time, '%k') AS hour,
                    {$conf['maintenance']['operationInterval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    diac.ad_id AS ad_id,
                    diac.creative_id AS creative_id,
                    diac.zone_id AS zone_id,
                    0 AS basket_value
                FROM
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac
                LEFT JOIN
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS diavv
                USING
                    (
                        data_intermediate_ad_connection_id
                    )
                LEFT JOIN
                    {$conf['table']['prefix']}{$conf['table']['variables']} AS v
                ON
                    (
                        diavv.tracker_variable_id = v.variableid
                    )
                WHERE
                    diac.connection_status = ". MAX_CONNECTION_STATUS_APPROVED ."
                    AND v.is_basket_value = 0
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
            if (count($aIds) > 0) {
                $query .= "
                    AND diac.data_intermediate_ad_connection_id NOT IN (" . implode(', ', $aIds) . ")";
            }
            $query .= "
                GROUP BY
                    diac.data_intermediate_ad_connection_id, day, hour, ad_id, creative_id, zone_id";
            MAX::debug('Selecting conversions with non-basket values', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Select all conversions that do NOT have variable values attached,
            // into the tmp_conversions table
            $query = "
                INSERT INTO
                    tmp_conversions
                    (
                        data_intermediate_ad_connection_id,
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        basket_value
                    )
                SELECT
                    diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,
                    DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(diac.tracker_date_time, '%k') AS hour,
                    {$conf['maintenance']['operationInterval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    diac.ad_id AS ad_id,
                    diac.creative_id AS creative_id,
                    diac.zone_id AS zone_id,
                    0 AS basket_value
                FROM
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac
                LEFT JOIN
                    {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS diavv
                USING
                    (
                        data_intermediate_ad_connection_id
                    )
                WHERE
                    diac.connection_status = ". MAX_CONNECTION_STATUS_APPROVED ."
                    AND diavv.data_intermediate_ad_connection_id IS NULL
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
            MAX::debug('Selecting conversions with no basket values', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Create a union table of the requests, impressions, clicks and conversions
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_union
                TYPE={$conf['table']['type']}
                SELECT
                    tar.day AS day,
                    tar.hour AS hour,
                    tar.operation_interval AS operation_interval,
                    tar.operation_interval_id AS operation_interval_id,
                    tar.interval_start AS interval_start,
                    tar.interval_end AS interval_end,
                    tar.ad_id AS ad_id,
                    tar.creative_id AS creative_id,
                    tar.zone_id AS zone_id,
                    tar.requests AS requests,
                    0 AS impressions,
                    0 AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_request AS tar
                UNION ALL
                SELECT
                    tai.day AS day,
                    tai.hour AS hour,
                    tai.operation_interval AS operation_interval,
                    tai.operation_interval_id AS operation_interval_id,
                    tai.interval_start AS interval_start,
                    tai.interval_end AS interval_end,
                    tai.ad_id AS ad_id,
                    tai.creative_id AS creative_id,
                    tai.zone_id AS zone_id,
                    0 AS requests,
                    tai.impressions AS impressions,
                    0 AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_impression AS tai
                UNION ALL
                SELECT
                    tac.day AS day,
                    tac.hour AS hour,
                    tac.operation_interval AS operation_interval,
                    tac.operation_interval_id AS operation_interval_id,
                    tac.interval_start AS interval_start,
                    tac.interval_end AS interval_end,
                    tac.ad_id AS ad_id,
                    tac.creative_id AS creative_id,
                    tac.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    tac.clicks AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_click AS tac
                UNION ALL
                SELECT
                    tc.day AS day,
                    tc.hour AS hour,
                    tc.operation_interval AS operation_interval,
                    tc.operation_interval_id AS operation_interval_id,
                    tc.interval_start AS interval_start,
                    tc.interval_end AS interval_end,
                    tc.ad_id AS ad_id,
                    tc.creative_id AS creative_id,
                    tc.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    0 AS clicks,
                    COUNT(*) AS conversions,
                    SUM(tc.basket_value) AS total_basket_value
                FROM
                    tmp_conversions AS tc
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            MAX::debug('Creating a union of the requests, impressions, clicks and conversions', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Drop the tmp_conversions table
            $this->tempTables->dropTempTable('tmp_conversions');
            $table = $conf['table']['prefix'] .
                     $conf['table']['data_intermediate_ad'];
            $query = "
                INSERT INTO
                    $table
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        requests,
                        impressions,
                        clicks,
                        conversions,
                        total_basket_value,
                        updated
                    )
                SELECT
                    tu.day AS day,
                    tu.hour AS hour,
                    tu.operation_interval AS operation_interval,
                    tu.operation_interval_id AS operation_interval_id,
                    tu.interval_start AS interval_start,
                    tu.interval_end AS interval_end,
                    tu.ad_id AS ad_id,
                    tu.creative_id AS creative_id,
                    tu.zone_id AS zone_id,
                    SUM(tu.requests) AS requests,
                    SUM(tu.impressions) AS impressions,
                    SUM(tu.clicks) AS clicks,
                    SUM(tu.conversions) AS conversions,
                    SUM(tu.total_basket_value) AS total_basket_value,
                    '".date('Y-m-d H:i:s')."'
                FROM
                    tmp_union AS tu
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            MAX::debug("Inserting the ad requests, impressions, clicks, and conversions into the $table table",
                       PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Drop the tmp_union table
            $this->tempTables->dropTempTable('tmp_union');
        } else {
            // Tracker module not installed, just summarise requests, impressions
            // and clicks: Create a union table of the requests, impressions and clicks
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_union
                TYPE={$conf['table']['type']}
                SELECT
                    tar.day AS day,
                    tar.hour AS hour,
                    tar.operation_interval AS operation_interval,
                    tar.operation_interval_id AS operation_interval_id,
                    tar.interval_start AS interval_start,
                    tar.interval_end AS interval_end,
                    tar.ad_id AS ad_id,
                    tar.creative_id AS creative_id,
                    tar.zone_id AS zone_id,
                    tar.requests AS requests,
                    0 AS impressions,
                    0 AS clicks
                FROM
                    tmp_ad_request AS tar
                UNION ALL
                SELECT
                    tai.day AS day,
                    tai.hour AS hour,
                    tai.operation_interval AS operation_interval,
                    tai.operation_interval_id AS operation_interval_id,
                    tai.interval_start AS interval_start,
                    tai.interval_end AS interval_end,
                    tai.ad_id AS ad_id,
                    tai.creative_id AS creative_id,
                    tai.zone_id AS zone_id,
                    0 AS requests,
                    tai.impressions AS impressions,
                    0 AS clicks
                FROM
                    tmp_ad_impression AS tai
                UNION ALL
                SELECT
                    tac.day AS day,
                    tac.hour AS hour,
                    tac.operation_interval AS operation_interval,
                    tac.operation_interval_id AS operation_interval_id,
                    tac.interval_start AS interval_start,
                    tac.interval_end AS interval_end,
                    tac.ad_id AS ad_id,
                    tac.creative_id AS creative_id,
                    tac.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    tac.clicks AS clicks
                FROM
                    tmp_ad_click AS tac
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            MAX::debug('Creating a union of the requests, impressions and clicks', PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            $table = $conf['table']['prefix'] .
                     $conf['table']['data_intermediate_ad'];
            $query = "
                INSERT INTO
                    $table
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        requests,
                        impressions,
                        clicks,
                        conversions,
                        total_basket_value,
                        updated
                    )
                SELECT
                    tu.day AS day,
                    tu.hour AS hour,
                    tu.operation_interval AS operation_interval,
                    tu.operation_interval_id AS operation_interval_id,
                    tu.interval_start AS interval_start,
                    tu.interval_end AS interval_end,
                    tu.ad_id AS ad_id,
                    tu.creative_id AS creative_id,
                    tu.zone_id AS zone_id,
                    SUM(tu.requests) AS requests,
                    SUM(tu.impressions) AS impressions,
                    SUM(tu.clicks) AS clicks,
                    '0' AS conversions,
                    '0' AS total_basket_value,
                    '".date('Y-m-d H:i:s')."'
                FROM
                    tmp_union AS tu
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            MAX::debug("Inserting the ad requests, impressions and clicks into the $table table", PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Drop the tmp_union table
            $this->tempTables->dropTempTable('tmp_union');
        }
        // Drop the tmp_ad_request table
        $this->tempTables->dropTempTable('tmp_ad_request');
        // Drop the tmp_ad_impression table
        $this->tempTables->dropTempTable('tmp_ad_impression');
        // Drop the tmp_ad_click table
        $this->tempTables->dropTempTable('tmp_ad_click');
    }

    /**
     * A private method to save the tacker impression/ad impression and
     * tracker impression/ad click connections that have been marked as
     * "latest", and also save the associated tracker impression
     * variable values.
     *
     * @protected
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function _saveConnectionsAndVariableValues($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        // Save the connections
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_intermediate_ad_connection'];
        $query = "
            INSERT INTO
                $table
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    viewer_id,
                    viewer_session_id,
                    tracker_date_time,
                    connection_date_time,
                    tracker_id,
                    ad_id,
                    creative_id,
                    zone_id,
                    tracker_channel,
                    connection_channel,
                    tracker_language,
                    connection_language,
                    tracker_ip_address,
                    connection_ip_address,
                    tracker_host_name,
                    connection_host_name,
                    tracker_country,
                    connection_country,
                    tracker_https,
                    connection_https,
                    tracker_domain,
                    connection_domain,
                    tracker_page,
                    connection_page,
                    tracker_query,
                    connection_query,
                    tracker_referer,
                    connection_referer,
                    tracker_search_term,
                    connection_search_term,
                    tracker_user_agent,
                    connection_user_agent,
                    tracker_os,
                    connection_os,
                    tracker_browser,
                    connection_browser,
                    connection_type,
                    connection_window,
                    connection_status,
                    updated
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.viewer_id AS viewer_id,
                drti.viewer_session_id AS viewer_session_id,
                drti.date_time AS tracker_date_time,
                tac.connection_date_time AS connection_date_time,
                drti.tracker_id AS tracker_id,
                tac.connection_ad_id AS ad_id,
                tac.connection_creative_id AS creative_id,
                tac.connection_zone_id AS zone_id,
                drti.channel AS tracker_channel,
                tac.connection_channel AS connection_channel,
                drti.language AS tracker_language,
                tac.connection_language AS connection_language,
                drti.ip_address AS tracker_ip_address,
                tac.connection_ip_address AS connection_ip_address,
                drti.host_name AS tracker_host_name,
                tac.connection_host_name AS connection_host_name,
                drti.country AS tracker_country,
                tac.connection_country AS connection_country,
                drti.https AS tracker_https,
                tac.connection_https AS connection_https,
                drti.domain AS tracker_domain,
                tac.connection_domain AS connection_domain,
                drti.page AS tracker_page,
                tac.connection_page AS connection_page,
                drti.query AS tracker_query,
                tac.connection_query AS connection_query,
                drti.referer AS tracker_referer,
                tac.connection_referer AS connection_referer,
                drti.search_term AS tracker_search_term,
                tac.connection_search_term AS connection_search_term,
                drti.user_agent AS tracker_user_agent,
                tac.connection_user_agent AS connection_user_agent,
                drti.os AS tracker_os,
                tac.connection_os AS connection_os,
                drti.browser AS tracker_browser,
                tac.connection_browser AS connection_browser,
                tac.connection_type AS connection_type,
                tac.connection_window AS connection_window,
                tac.connection_status AS connection_status,
                '".date('Y-m-d H:i:s')."'
            FROM
                tmp_ad_connection AS tac,
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']} AS drti
            WHERE
                tac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                AND tac.server_raw_ip = drti.server_raw_ip
                AND tac.latest = 1";
        MAX::debug("Inserting the connections into the $table table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        // Save the variable values
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_intermediate_ad_variable_value'];
        $query = "
            INSERT INTO
                $table
                (
                    data_intermediate_ad_connection_id,
                    tracker_variable_id,
                    value
                )
            SELECT
                diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,
                drtvv.tracker_variable_id AS tracker_variable_id,
                drtvv.value AS value
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac,
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_variable_value']} AS drtvv,
                {$conf['table']['prefix']}{$conf['table']['data_raw_tracker_impression']} AS drti
            WHERE
                diac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                AND diac.server_raw_ip = drti.server_raw_ip
                AND drti.server_raw_tracker_impression_id = drtvv.server_raw_tracker_impression_id
                AND drti.server_raw_ip = drtvv.server_raw_ip
                AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
        MAX::debug("Saving the tracker impression variable values from the $table table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        // Remove from diac all duplicates
        $this->_dedupConversions($start, $end);
    }

    /**
     * A private method to dedup conversions which have associated unique variables.
     * The method check connections from last interval (between $start and $end) and dedup them
     * between those dates.
     *
     * @param Date $start The start date/time of current interval
     * @param Date $end The end date/time of current interval
     */
    function _dedupConversions($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];

        MAX::debug('Deduping conversions between "'.$start->format('%Y-%m-%d %H:%M:%S').'" and "'.$end->format('%Y-%m-%d %H:%M:%S').'"', PEAR_LOG_DEBUG);

        $query = "
            UPDATE
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac
            JOIN
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS diavv
            ON
                (
                    diac.data_intermediate_ad_connection_id = diavv.data_intermediate_ad_connection_id
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'
                )
            JOIN
                {$conf['table']['prefix']}{$conf['table']['variables']} AS v
            ON
                (
                    diavv.tracker_variable_id = v.variableid
                )
            JOIN
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_connection']} AS diac2
            ON
                (
                    v.trackerid = diac2.tracker_id
                    AND UNIX_TIMESTAMP(diac.tracker_date_time) - UNIX_TIMESTAMP(diac2.tracker_date_time) < v.unique_window
                    AND UNIX_TIMESTAMP(diac.tracker_date_time) - UNIX_TIMESTAMP(diac2.tracker_date_time) > 0
                    AND diac.viewer_id = diac2.viewer_id
                )
            JOIN
                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad_variable_value']} AS diavv2
            ON
                (
                    diac2.data_intermediate_ad_connection_id = diavv2.data_intermediate_ad_connection_id
                )
            JOIN
                {$conf['table']['prefix']}{$conf['table']['variables']} AS v2
            ON
                (
                    diavv2.tracker_variable_id = v2.variableid
                )
            SET diac.connection_status = ". MAX_CONNECTION_STATUS_DUPLICATE .",
                diac.updated = '". date('Y-m-d H:i:s') ."'
            WHERE
                v.is_unique = 1
        ";
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
    }

    /**
     * A method to update the zone impression history table from the intermediate tables.
     *
     * @param Date $start The start date/time to update from.
     * @param Date $end The end date/time to update to.
     */
    function saveHistory($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $fromTable = $conf['table']['prefix'] .
                     $conf['table']['data_intermediate_ad'];
        $toTable   = $conf['table']['prefix'] .
                     $conf['table']['data_summary_zone_impression_history'];
        $query = "
            SELECT
                operation_interval AS operation_interval,
                operation_interval_id AS operation_interval_id,
                interval_start AS interval_start,
                interval_end AS interval_end,
                zone_id AS zone_id,
                SUM(impressions) AS actual_impressions
            FROM
                $fromTable
            WHERE
                interval_start >= '".$start->format('%Y-%m-%d %H:%M:%S')."'
                AND interval_end <= '".$end->format('%Y-%m-%d %H:%M:%S')."'
            GROUP BY
                zone_id";
        MAX::debug('Selecting total zone impressions from the ' . $fromTable . ' table for data >= ' .
                   $start->format('%Y-%m-%d %H:%M:%S') . ', and <= ' . $end->format('%Y-%m-%d %H:%M:%S'),
                   PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        while ($row = $result->fetchRow()) {
            $query = "
                UPDATE
                    $toTable
                SET
                    actual_impressions = {$row['actual_impressions']}
                WHERE
                    operation_interval = {$row['operation_interval']}
                    AND operation_interval_id = {$row['operation_interval_id']}
                    AND interval_start = '{$row['interval_start']}'
                    AND interval_end = '{$row['interval_end']}'
                    AND zone_id = {$row['zone_id']}";
            $innerResult = $this->dbh->query($query);
            if (PEAR::isError($innerResult)) {
                return MAX::raiseError($innerResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            if ($this->dbh->affectedRows() == 0) {
                // Unable to UPDATE, try INSERT instead
                $query = "
                    INSERT INTO
                        $toTable
                        (
                            operation_interval,
                            operation_interval_id,
                            interval_start,
                            interval_end,
                            zone_id,
                            actual_impressions
                        )
                    VALUES
                        (
                            {$row['operation_interval']},
                            {$row['operation_interval_id']},
                            '{$row['interval_start']}',
                            '{$row['interval_end']}',
                            {$row['zone_id']},
                            {$row['actual_impressions']}
                        )";
                $innerResult = $this->dbh->query($query);
                if (PEAR::isError($innerResult)) {
                    return MAX::raiseError($innerResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                }
            }
        }
    }

    /**
     * A method to update the summary table from the intermediate tables.
     *
     * @param Date $oStartDate The start date/time to update from.
     * @param Date $oEndDate The end date/time to update to.
     */
    function saveSummary($oStartDate, $oEndDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $fromTable = $conf['table']['prefix'] .
                     $conf['table']['data_intermediate_ad'];
        $toTable   = $conf['table']['prefix'] .
                     $conf['table']['data_summary_ad_hourly'];
        // How many days does the start/end period span?
        $days = Date_Calc::dateDiff($oStartDate->getDay(),
                                    $oStartDate->getMonth(),
                                    $oStartDate->getYear(),
                                    $oEndDate->getDay(),
                                    $oEndDate->getMonth(),
                                    $oEndDate->getYear());
        if ($days == 0) {
            $query = "
                INSERT INTO
                    $toTable
                    (
                        day,
                        hour,
                        ad_id,
                        creative_id,
                        zone_id,
                        requests,
                        impressions,
                        clicks,
                        conversions,
                        total_basket_value,
                        updated
                    )
                SELECT
                    day AS day,
                    hour AS hour,
                    ad_id AS ad_id,
                    creative_id AS creative_id,
                    zone_id AS zone_id,
                    SUM(requests) AS requests,
                    SUM(impressions) AS impressions,
                    SUM(clicks) AS clicks,
                    SUM(conversions) AS conversions,
                    SUM(total_basket_value) AS total_basket_value,
                    '".date('Y-m-d H:i:s')."'
                FROM
                    $fromTable
                WHERE
                    day >= '".$oStartDate->format('%Y-%m-%d')."'
                    AND hour >= ".$oStartDate->format('%H')."
                    AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                    AND hour <= ".$oEndDate->format('%H')."
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            MAX::debug('Summarising ad requests, impressions, clicks and conversions into the ' . $toTable .
                       ' table from the ' . $fromTable . ' table for data >= day ' . $oStartDate->format('%Y-%m-%d') .
                       ', hour ' . $oStartDate->format('%H') . ', and <= day ' . $oEndDate->format('%Y-%m-%d') .
                       ', hour ' . $oEndDate->format('%H'), PEAR_LOG_DEBUG);
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            // Update the recently summarised data with basic financial information
            $this->_updateWithFinanceInfo($oStartDate, $oEndDate);
        } else {
            // Update each day in turn
            for ($counter = 0; $counter <= $days; $counter++) {
                if ($counter == 0) {
                    // This is the first day
                    $oInternalStartDate = &new Date();
                    $oInternalStartDate->copy($oStartDate);
                    $oInternalEndDate = &new Date($oStartDate->format('%Y-%m-%d') . ' 23:59:59');
                } elseif ($counter == $days) {
                    // This is the last day
                    $oInternalStartDate = &new Date($oEndDate->format('%Y-%m-%d') . ' 00:00:00');
                    $oInternalEndDate = &new Date();
                    $oInternalEndDate->copy($oEndDate);
                } else {
                    // This is a day in the middle
                    $oDayDate = &new Date();
                    $oDayDate->copy($oStartDate);
                    $oDayDate->addSeconds(SECONDS_PER_DAY * $counter);
                    $oInternalStartDate = &new Date($oDayDate->format('%Y-%m-%d') . ' 00:00:00');
                    $oInternalEndDate = &new Date($oDayDate->format('%Y-%m-%d') . ' 23:59:59');
                }
                $query = "
                    INSERT INTO
                        $toTable
                        (
                            day,
                            hour,
                            ad_id,
                            creative_id,
                            zone_id,
                            requests,
                            impressions,
                            clicks,
                            conversions,
                            total_basket_value
                        )
                    SELECT
                        day AS day,
                        hour AS hour,
                        ad_id AS ad_id,
                        creative_id AS creative_id,
                        zone_id AS zone_id,
                        SUM(requests) AS requests,
                        SUM(impressions) AS impressions,
                        SUM(clicks) AS clicks,
                        SUM(conversions) AS conversions,
                        SUM(total_basket_value) AS total_basket_value
                    FROM
                        $fromTable
                    WHERE
                        day >= '".$oInternalStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oInternalStartDate->format('%H')."
                        AND day <= '".$oInternalEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oInternalEndDate->format('%H')."
                    GROUP BY
                        day, hour, ad_id, creative_id, zone_id";
                MAX::debug('Summarising ad requests, impressions, clicks and conversions into the ' . $toTable .
                           ' table from the ' . $fromTable . ' table for data >= day ' .
                           $oInternalStartDate->format('%Y-%m-%d') . ', hour ' . $oInternalStartDate->format('%H') .
                           ', and <= day ' . $oInternalEndDate->format('%Y-%m-%d') . ', hour ' .
                           $oInternalEndDate->format('%H'), PEAR_LOG_DEBUG);
                $result = $this->dbh->query($query);
                if (PEAR::isError($result)) {
                    return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                }
                // Update the recently summarised data with basic financial information
                $this->_updateWithFinanceInfo($oInternalStartDate, $oInternalEndDate);
            }
        }
    }

    /**
     * A method to activate/deactivate campaigns, based on the date and/or the inventory
     * requirements (impressions, clicks and/or conversions).
     *
     * @param Date $oDate The current date/time.
     * @return string Report on the campaigns activated/deactivated.
     */
    function manageCampaigns($oDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $report .= "\n";
        $query = "
            SELECT
                ca.campaignid AS campaign_id,
                ca.campaignname AS campaign_name,
                cl.contact AS contact,
                cl.email AS email,
                ca.views AS targetImpressions,
                ca.clicks AS targetClicks,
                ca.conversions AS targetConversions,
                ca.active AS active,
                ca.activate AS start,
                ca.expire AS end,
                NOW() AS now
            FROM
                {$conf['table']['prefix']}{$conf['table']['campaigns']} AS ca,
                {$conf['table']['prefix']}{$conf['table']['clients']} AS cl
            WHERE
                ca.clientid = cl.clientid";
        MAX::debug("Selecting all campaigns", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        while ($campaignRow = $result->fetchRow()) {
            if ($campaignRow['active'] == 't') {
                // The campaign is currently active, look at the campaign
                $disableReason = 0;
                if (($campaignRow['targetImpressions'] > 0) ||
                    ($campaignRow['targetClicks'] > 0) ||
                    ($campaignRow['targetConversions'] > 0)) {
                    // The campaign has an impression, click and/or conversion target,
                    // so get the sum total statistics for the campaign
                    $query = "
                        SELECT
                            SUM(dia.impressions) AS impressions,
                            SUM(dia.clicks) AS clicks,
                            SUM(dia.conversions) AS conversions
                        FROM
                            {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad']} AS dia,
                            {$conf['table']['prefix']}{$conf['table']['banners']} AS b
                        WHERE
                            dia.ad_id = b.bannerid
                            AND b.campaignid = {$campaignRow['campaign_id']}";
                    $valuesRow = $this->dbh->getRow($query);
                    if ((!is_null($valuesRow['impressions'])) || (!is_null($valuesRow['clicks'])) || (!is_null($valuesRow['conversions']))) {
                        // There were impressions, clicks and/or conversions for this
                        // campaign, so find out if campaign targets have been passed
                        if (is_null($valuesRow['impressions'])) {
                            // No impressions
                            $valuesRow['impressions'] = 0;
                        }
                        if (is_null($valuesRow['clicks'])) {
                            // No clicks
                            $valuesRow['clicks'] = 0;
                        }
                        if (is_null($valuesRow['conversions'])) {
                            // No conversions
                            $valuesRow['conversions'] = 0;
                        }
                        if ($campaignRow['targetImpressions'] > 0) {
                            if ($campaignRow['targetImpressions'] <= $valuesRow['impressions']) {
                                // The campaign has an impressions target, and this has been
                                // passed, so update and disable the campagin
                                $disableReason |= MAX_PLACEMENT_DISABLED_IMPRESSIONS;
                            }
                        }
                        if ($campaignRow['targetClicks'] > 0) {
                            if ($campaignRow['targetClicks'] <= $valuesRow['clicks']) {
                                // The campaign has a click target, and this has been
                                // passed, so update and disable the campaign
                                $disableReason |= MAX_PLACEMENT_DISABLED_CLICKS;
                            }
                        }
                        if ($campaignRow['targetConversions'] > 0) {
                            if ($campaignRow['targetConversions'] <= $valuesRow['conversions']) {
                                // The campaign has a target limitation, and this has been
                                // passed, so update and disable the campagin
                                $disableReason |= MAX_PLACEMENT_DISABLED_CONVERSIONS;
                            }
                        }
                        if ($disableReason) {
                            // One of the campaign targets was exceeded, so disable
                            $query = "
                                UPDATE
                                    {$conf['table']['prefix']}{$conf['table']['campaigns']}
                                SET
                                    active = 'f'
                                WHERE
                                    campaignid = {$campaignRow['campaign_id']}";
                            $report .= ' - Exceeded a campaign quota: Deactivating campaign ID ' .
                                  "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                            MAX::debug('Exceeded a campaign quota: Deactivating campaign ID ' .
                                       "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}");
                            $innerResult = $this->dbh->query($query);
                            if (PEAR::isError($innerResult)) {
                                return MAX::raiseError($innerResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                            }
                            phpAds_userlogAdd(phpAds_actionDeactiveCampaign, $campaignRow['campaign_id']);
                        }
                    }
                }
                // Does the campaign need to be disabled due to the date?
                $end = new Date($campaignRow['end'] . ' 23:59:59');  // Convert day to end of day Date
                if (($end->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 23:59:59') && ($oDate->after($end))) {
                    $disableReason |= MAX_PLACEMENT_DISABLED_DATE;
                    $query = "
                        UPDATE
                            {$conf['table']['prefix']}{$conf['table']['campaigns']}
                        SET
                            active = 'f'
                        WHERE
                            campaignid = {$campaignRow['campaign_id']}";
                    $report .= ' - Past campaign end time: Deactivating campaign ID ' .
                          "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                    MAX::debug('Found campaign end date of ' . $end->format('%Y-%m-%d %H:%M:%S') . ' has been '.
                               'passed by current time of ' . $oDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
                    MAX::debug('Passed campaign end time: Deactivating campaign ID ' .
                               "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}", PEAR_LOG_INFO);
                    $innerResult = $this->dbh->query($query);
                    if (PEAR::isError($innerResult)) {
                        return MAX::raiseError($innerResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                    }
                    phpAds_userlogAdd(phpAds_actionDeactiveCampaign, $campaignRow['campaign_id']);
                }
                if ($disableReason) {
                    // The campaign was disabled, so send the appropriate
                    // message to the campaign's contact
                    $query = "
                        SELECT
                            bannerid AS advertisement_id,
                            description AS description,
                            alt AS alt,
                            url AS url
                        FROM
                            {$conf['table']['prefix']}{$conf['table']['banners']}
                        WHERE
                            campaignid = {$campaignRow['campaign_id']}";
                    MAX::debug("Getting the advertisements for campaign ID {$campaignRow['campaign_id']}", PEAR_LOG_DEBUG);
                    $advertisementResult = $this->dbh->query($query);
                    if (PEAR::isError($advertisementResult)) {
                        return MAX::raiseError($advertisementResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                    }
                    while ($advertisementRow = $advertisementResult->fetchRow()) {
                        $advertisements[$advertisementRow['advertisement_id']] =
                            array($advertisementRow['description'], $advertisementRow['alt'],
                                  $advertisementRow['url']);
                    }
                    if ($conf['email']['sendMail']) {
                        $message =& $this->maintenance->prepareDeactivateCampaignEmail($campaignRow['contact'],
                                                                                $campaignRow['campaign_name'],
                                                                                $disableReason, $advertisements);
                        MAX::sendMail($campaignRow['email'], $campaignRow['contact'],
                                      "Deactivated Banners: {$campaignRow['campaign_name']}", $message);
                    }
                }
            } else {
                // The campaign is not active - does it need to be enabled,
                // based on the campaign starting date?
                $start = new Date($campaignRow['start'] . ' 00:00:00');  // Convert day to start of date Date
                $end   = new Date($campaignRow['end']   . ' 23:59:59');  // Convert day to end of day Date
                if (($start->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 00:00:00') && ($oDate->after($start))) {
                    // There is an activation date, which has been passed. Find out if
                    // there are any impression, click or conversion targets for the
                    // placement (i.e. if the target values are > 0)
                    $remainingImpressions = 0;
                    $remainingClicks      = 0;
                    $remainingConversions = 0;
                    if (($campaignRow['targetImpressions'] > 0) ||
                        ($campaignRow['targetClicks'] > 0) ||
                        ($campaignRow['targetConversions'] > 0)) {
                        // The placement has an impression, click and/or conversion target,
                        // so get the sum total statistics for the placement so far
                        $query = "
                            SELECT
                                SUM(dia.impressions) AS impressions,
                                SUM(dia.clicks) AS clicks,
                                SUM(dia.conversions) AS conversions
                            FROM
                                {$conf['table']['prefix']}{$conf['table']['data_intermediate_ad']} AS dia,
                                {$conf['table']['prefix']}{$conf['table']['banners']} AS b
                            WHERE
                                dia.ad_id = b.bannerid
                                AND b.campaignid = {$campaignRow['campaign_id']}";
                        $valuesRow = $this->dbh->getRow($query);
                        // Set the remaining impressions, clicks and conversions for the placement
                        $remainingImpressions = $campaignRow['targetImpressions'] - $valuesRow['impressions'];
                        $remainingClicks      = $campaignRow['targetClicks']      - $valuesRow['clicks'];
                        $remainingConversions = $campaignRow['targetConversions'] - $valuesRow['conversions'];
                    }
                    // In order for the placement to be activated, need to test:
                    // 1) That there is no impression target (<=0), or, if there is an impression target (>0),
                    //    then there must be remaining impressions to deliver (>0); and
                    // 2) That there is no click target (<=0), or, if there is a click target (>0),
                    //    then there must be remaining clicks to deliver (>0); and
                    // 3) That there is no conversion target (<=0), or, if there is a conversion target (>0),
                    //    then there must be remaining conversions to deliver (>0); and
                    // 4) Either there is no end date, or the end date has not been passed
                    if ((($campaignRow['targetImpressions'] <= 0) || (($campaignRow['targetImpressions'] > 0) && ($remainingImpressions > 0))) &&
                        (($campaignRow['targetClicks']      <= 0) || (($campaignRow['targetClicks']      > 0) && ($remainingClicks      > 0))) &&
                        (($campaignRow['targetConversions'] <= 0) || (($campaignRow['targetConversions'] > 0) && ($remainingConversions > 0))) &&
                        (($end->format('%Y-%m-%d %H:%M:%S') == '0000-00-00 00:00:00') || (($end->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 00:00:00') && (Date::compare($oDate, $end) < 0)))) {
                        $query = "
                            UPDATE
                                {$conf['table']['prefix']}{$conf['table']['campaigns']}
                            SET
                                active = 't'
                            WHERE
                                campaignid = {$campaignRow['campaign_id']}";
                        $report .= ' - Past campaign start time: Activating campaign ID ' .
                              "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                        MAX::debug('Found campaign start date of ' . $start->format('%Y-%m-%d %H:%M:%S') . ' has been '.
                                   'passed by current time of ' . $oDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
                        MAX::debug('Passed campaign start time: Activating campaign ID ' .
                                   "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}", PEAR_LOG_INFO);
                        $innerResult = $this->dbh->query($query);
                        if (PEAR::isError($innerResult)) {
                            return MAX::raiseError($innerResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                        }
                        phpAds_userlogAdd(phpAds_actionActiveCampaign, $campaignRow['campaign_id']);
                        // Get the advertisements associated with the campaign
                        $query = "
                            SELECT
                                bannerid AS advertisement_id,
                                description AS description,
                                alt AS alt,
                                url AS url
                            FROM
                                {$conf['table']['prefix']}{$conf['table']['banners']}
                            WHERE
                                campaignid = {$campaignRow['campaign_id']}";
                        MAX::debug("Getting the advertisements for campaign ID {$campaignRow['campaign_id']}",
                                   PEAR_LOG_DEBUG);
                        $advertisementResult = $this->dbh->query($query);
                        if (PEAR::isError($advertisementResult)) {
                            return MAX::raiseError($advertisementResult, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
                        }
                        while ($advertisementRow = $advertisementResult->fetchRow()) {
                            $advertisements[$advertisementRow['advertisement_id']] =
                                array($advertisementRow['description'], $advertisementRow['alt'],
                                    $advertisementRow['url']);
                        }
                        if ($conf['email']['sendMail']) {
                            $message =& $this->maintenance->prepareActivateCampaignEmail($campaignRow['contact'],
                                                                                  $campaignRow['campaign_name'],
                                                                                  $advertisements);
                            MAX::sendMail($campaignRow['email'], $campaignRow['contact'],
                                          "Activated Banners: {$campaignRow['campaign_name']}", $message);
                        }
                    }
                }
            }
        }
    }

    /**
     * A method to delete old (ie. summarised) raw data.
     *
     * @param Date $summarisedTo The date/time up to which data have been summarised (i.e. data up
     *                           to and including this date (minus any compact_stats_grace window)
     *                           will be deleted, unless required by the tracking module, where
     *                           installed).
     * @return integer The number of rows deleted.
     */
    function deleteOldData($summarisedTo)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $deleteDate = $summarisedTo;
        if ($conf['maintenance']['compactStatsGrace'] > 0) {
            $deleteDate->subtractSeconds((int) $conf['maintenance']['compactStatsGrace']);
        }
        $rows = 0;
        // Delete the ad requests before taking into account the maximum connection window
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_raw_ad_request'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        MAX::debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                   "') ad requests from the $table table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows += $this->dbh->affectedRows();
        // Take into account the maximum connection window, if approproate
        if ($conf['modules']['Tracker']) {
            // Find the largest, active impression and click connection windows
            list($impressionWindow, $clickWindow) = $this->maintenance->maxConnectionWindows();
            // Find the largest of the two windows
            if ($impressionWindow > $clickWindow) {
                $maxWindow = $impressionWindow;
            } else {
                $maxWindow = $clickWindow;
            }
            MAX::debug('Found maximum connection window of ' . $maxWindow . ' seconds', PEAR_LOG_DEBUG);
            $deleteDate->subtractSeconds((int) $maxWindow); // Cast to int, as Date class
                                                            // doesn't deal with strings
        }
        // Delete the ad impressions
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_raw_ad_impression'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        MAX::debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                   "') ad impressions from the $table table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows += $this->dbh->affectedRows();
        // Delete the ad clicks
        $table = $conf['table']['prefix'] .
                 $conf['table']['data_raw_ad_click'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        MAX::debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                   "') ad clicks from the $table table", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $rows += $this->dbh->affectedRows();
        return $rows;
    }

    /**
     * A method to set basic financial information in the data_summary_ad_hourly table,
     * on the basis of placement and zone financial information.
     *
     * @access private
     *
     * @param Date $oStartDate The start date for records that need updating.
     * @param Date $oEndDate The end date for records that need updating.
     *
     * @TODO Remove "return true" once fix to #494 is found!
     */
    function _updateWithFinanceInfo($oStartDate, $oEndDate)
    {
        return true;
        $conf = $GLOBALS['_MAX']['CONF'];
        // Obtain the list of ads with basic financial information
        $query = "
            SELECT DISTINCT
                ad_id AS ad_id
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
            WHERE
                day >= '".$oStartDate->format('%Y-%m-%d')."'
                AND hour >= ".$oStartDate->format('%H')."
                AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                AND hour <= ".$oEndDate->format('%H');
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $aAdIds = array();
        while ($aRow = $result->fetchRow()) {
            $aAdIds[] = $aRow['ad_id'];
        }
        $aAdInfo = $this->_getAdsWithFinanceInfo($aAdIds);
        // Update the recently summarised data with basic financial information
        if ($aAdInfo) {
            $this->_updateAdsWithFinanceInfo($aAdInfo, $oStartDate, $oEndDate);
        }
        // Obtain the list of zones with basic financial information
        $query = "
            SELECT DISTINCT
                zone_id AS zone_id
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
            WHERE
                day >= '".$oStartDate->format('%Y-%m-%d')."'
                AND hour >= ".$oStartDate->format('%H')."
                AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                AND hour <= ".$oEndDate->format('%H');
        $result = $this->dbh->query($query);
        if (PEAR::isError($result)) {
            return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
        }
        $aZoneIds = array();
        while ($aRow = $result->fetchRow()) {
            $aZoneIds[] = $aRow['zone_id'];
        }
        $aZoneInfo = $this->_getZonesWithFinanceInfo($aZoneIds);
        // Update the recently summarised data with basic financial information
        if ($aZoneInfo) {
            $this->_updateZonesWithFinanceInfo($aZoneInfo, $oStartDate, $oEndDate);
        }
    }

    /**
     * A method to obtain all ads, financial information, from placements that have basic financial
     * information set.
     *
     * @access private
     *
     * @param array $aAdIds An array of ad IDs that will be used to limit which ads to look at.
     *
     * @return mixed An array of arrays, each containing the ad_id, revenue and revenue_type of any
     *               matching ads, or false if none.
     */
    function _getAdsWithFinanceInfo($aAdIds)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (is_array($aAdIds) && (count($aAdIds) > 0)) {
            $query = "
                SELECT
                    a.bannerid AS ad_id,
                    c.revenue AS revenue,
                    c.revenue_type AS revenue_type
                FROM
                    {$conf['table']['prefix']}{$conf['table']['campaigns']} AS c,
                    {$conf['table']['prefix']}{$conf['table']['banners']} AS a
                WHERE
                    a.bannerid IN (" . implode(', ', $aAdIds) . ")
                    AND a.campaignid = c.campaignid
                    AND c.revenue IS NOT NULL
                    AND c.revenue_type IS NOT NULL";
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            $aResult = array();
            while ($aRow = $result->fetchRow()) {
                $aResult[] = $aRow;
            }
            if (count($aResult) > 0) {
                return $aResult;
            }
        }
        return false;
    }

    /**
     * A method to set basic financial information in the data_summary_ad_hourly table,
     * on the basis of ad financial information.
     *
     * @access private
     *
     * @param array $aAdInfo An array of arrays, each with the ad id, revenue and revenue type information.
     * @param Date $oStartDate The start date for records that need updating.
     * @param Date $oEndDate The end date for records that need updating.
     *
     * @TODO Update to deal with monthly tenancy.
     */
    function _updateAdsWithFinanceInfo($aAdInfo, $oStartDate, $oEndDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        foreach ($aAdInfo as $aInfo) {
            if ($aInfo['revenue_type'] == MAX_FINANCE_CPM) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_revenue = (impressions / 1000) * {$aInfo['revenue']}
                    WHERE
                        ad_id = {$aInfo['ad_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['revenue_type'] == MAX_FINANCE_CPC) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_revenue = clicks * {$aInfo['revenue']}
                    WHERE
                        ad_id = {$aInfo['ad_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['revenue_type'] == MAX_FINANCE_CPA) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_revenue = conversions * {$aInfo['revenue']}
                    WHERE
                        ad_id = {$aInfo['ad_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['revenue_type'] == MAX_FINANCE_MT) {
                break;
            }
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
        }
    }

    /**
     * A method to obtain all zones, financial information, from zones that have basic financial
     * information set.
     *
     * @access private
     *
     * @param array $aZoneIds An array of zone IDs that will be used to limit which zones to look at.
     *
     * @return mixed An array of arrays, each containing the zone_id, cost and cost_type of any
     *               matching zones, or false if none.
     */
    function _getZonesWithFinanceInfo($aZoneIds)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (is_array($aZoneIds) && (count($aZoneIds) > 0)) {
            $query = "
                SELECT
                    z.zoneid AS zone_id,
                    z.cost AS cost,
                    z.cost_type AS cost_type
                FROM
                    {$conf['table']['prefix']}{$conf['table']['zones']} AS z
                WHERE
                    z.zoneid IN (" . implode(', ', $aZoneIds) . ")
                    AND z.cost IS NOT NULL
                    AND z.cost_type IS NOT NULL";
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            $aResult = array();
            while ($aRow = $result->fetchRow()) {
                $aResult[] = $aRow;
            }
            if (count($aResult) > 0) {
                return $aResult;
            }
        }
        return false;
    }

    /**
     * A method to set basic financial information in the data_summary_ad_hourly table,
     * on the basis of zone financial information.
     *
     * @access private
     *
     * @param array $aZoneInfo An array of arrays, each with the zone id, cost and cost type information.
     * @param Date $oStartDate The start date for records that need updating.
     * @param Date $oEndDate The end date for records that need updating.
     *
     * @TODO Update to deal with monthly tenancy.
     */
    function _updateZonesWithFinanceInfo($aZoneInfo, $oStartDate, $oEndDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        foreach ($aZoneInfo as $aInfo) {
            if ($aInfo['cost_type'] == MAX_FINANCE_CPM) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_cost = (impressions / 1000) * {$aInfo['cost']}
                    WHERE
                        zone_id = {$aInfo['zone_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['cost_type'] == MAX_FINANCE_CPC) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_cost = clicks * {$aInfo['cost']}
                    WHERE
                        zone_id = {$aInfo['zone_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['cost_type'] == MAX_FINANCE_CPA) {
                $query = "
                    UPDATE
                        {$conf['table']['prefix']}{$conf['table']['data_summary_ad_hourly']}
                    SET
                        total_cost = conversions * {$aInfo['cost']}
                    WHERE
                        zone_id = {$aInfo['zone_id']}
                        AND day >= '".$oStartDate->format('%Y-%m-%d')."'
                        AND hour >= ".$oStartDate->format('%H')."
                        AND day <= '".$oEndDate->format('%Y-%m-%d')."'
                        AND hour <= ".$oEndDate->format('%H');
            } elseif ($aInfo['cost_type'] == MAX_FINANCE_MT) {
                break;
            }
            $result = $this->dbh->query($query);
            if (PEAR::isError($result)) {
                return MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
        }
    }

}

?>
