<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: cookie.php 4439 2006-03-13 15:10:11Z chris@m3.net $
*/

/**
 * @package    MaxDelivery
 * @subpackage cookie
 * @author     Chris Nutting <chris@m3.net>
 */


/**
 * Set a cookie in the global cookie cache
 * 
 * @param string $name name of cookie to be set
 * @param string $value value to be stored in the cookie
 * @param string $expire timestamp at which the cookie should be set to expire
 * 
 */
function MAX_setCookie($name, $value, $expire = 0)
{
    $cookieCache =& $GLOBALS['_MAX']['COOKIE'];
    if (!isset($cookieCache)) {
        $cookieCache = array();
    }
    $cookieCache[$name] = array($value, $expire);
}

function MAX_setUserIDCookieAndRedirect($viewerId) {
    $conf = $GLOBALS['_MAX']['CONF'];
    
    MAX_setCookie($conf['var']['viewerId'], $viewerId, time()+365*24*60*60);
    MAX_flushCookie();
    
    // Determine if the access to Max was made using HTTPS
    if ($_SERVER['SERVER_PORT'] == $conf['max']['sslPort']) {
        $url = constructSecureDeliveryUrl(basename($_SERVER['PHP_SELF']));
    } else {
        $url = constructDeliveryUrl(basename($_SERVER['PHP_SELF']));
    }
    $url .= "?{$conf['var']['cookieTest']}=1&" . $_SERVER['QUERY_STRING'];
    header("Location: {$url}");
    exit;
}

/**
 * Send all cookies in the global cookie cache to the browser
 * 
 */ 
function MAX_flushCookie()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    $cookieCache =& $GLOBALS['_MAX']['COOKIE'];
    if (isset($cookieCache)) {
        // Send P3P headers
        if ($conf['p3p']['policies']) {
            $p3p_header = '';
            if ($conf['p3p']['policyLocation'] != '') {
                $p3p_header .= " policyref=\"".$conf['p3p']['policyLocation']."\"";
            }
            if ($conf['p3p']['policyLocation'] != '' && $conf['p3p']['compactPolicy'] != '') {
                $p3p_header .= ", ";
            }
            if ($conf['p3p']['compactPolicy'] != '') {
                $p3p_header .= " CP=\"".$conf['p3p']['compactPolicy']."\"";
            }
            if ($p3p_header != '') {
                header("P3P: $p3p_header");
            }
        }
        // Set cookies
        while (list($name,$v) = each ($cookieCache)) {
            list($value, $expire) = $v;
            setcookie($name, $value, $expire, '/', (!empty($conf['cookie']['domain']) ? $conf['cookie']['domain'] : null));
        }
        // Clear cache
        $cookieCache = array();
    }
}

/**
 * This function gets the unique user ID (creating if necessary)
 *
 * @param bool $create Should a viewer ID be created if not present in $_COOKIE ?
 * @return string The viewer ID
 */
function MAX_getUniqueUserID($create = true)
{
    $conf = &$GLOBALS['_MAX']['CONF'];
    if (isset($_COOKIE[$conf['var']['viewerId']])) {
        $userid = $_COOKIE[$conf['var']['viewerId']];
    } else {
        if ($create) {
            $remote_address = $_SERVER['REMOTE_ADDR'];
            $local_address  = $conf['webpath']['delivery']; // How do I get the IP address of this server?
            // Get the exact time
            list($usec, $sec) = explode(" ", microtime());
            $time = (float) $usec + (float) $sec;
            // Get a random number
            $random = mt_rand(0,999999999);
            $userid = substr(md5($local_address.$time.$remote_address.$random),0,32);  // Need to find a way to generate this...
            $GLOBALS['_MAX']['newViewerId'] = true;
        } else {
            $userid = null;
        }
    }
    return $userid;
}

/**
 * This function generates a psudo-unique user ID based on IP address and user-agent
 *
 * @return string The cookieless viewer ID
 */
function MAX_getCookielessUserID()
{
    $remote_address = $_SERVER['REMOTE_ADDR'];
    $user_agent = (!empty($_SERVER['HTTP_USER_AGENT'])) ? $_SERVER['HTTP_USER_AGENT'] : '';
    //$http_accept = (!empty($_SERVER['HTTP_ACCEPT'])) ? $_SERVER['HTTP_ACCEPT'] : '';
    $http_accept_language = (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) ? $_SERVER['HTTP_ACCEPT_LANGUAGE'] : '';
    $http_accept_encoding = (!empty($_SERVER['HTTP_ACCEPT_ENCODING'])) ? $_SERVER['HTTP_ACCEPT_ENCODING'] : '';
    return MAX_COOKIELESS_PREFIX . substr(md5($remote_address.$user_agent.$http_accept_language.$http_accept_encoding), 0, 32-(strlen(MAX_COOKIELESS_PREFIX)));
   
}
?>
