<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Maintenance.mtc.test.php 3401 2005-06-23 13:01:54Z andrew $
*/

require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once MAX_PATH . '/lib/max/Maintenance/Forecasting/Channel/ChannelLimitations.php';
require_once MAX_PATH . '/lib/max/Table/Core.php';
require_once 'Date.php';

/**
 * A class for testing the ChannelLimitations class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     Radek Maciaszek <radek@m3.net>
 */
class Maintenance_TestOfChannelLimitations extends UnitTestCase
{

    var $errorLevel;
    
    function setUp()
    {
        $this->errorLevel = error_reporting(E_ALL);
    }
    
    function tearDown()
    {
        error_reporting($this->errorLevel);
    }
    
    /**
     * The constructor method.
     */    
    function Maintenance_TestOfChannelLimitations()
    {        
        $this->UnitTestCase();
        
        Mock::generate('MAX_Maintenance_Forecasting_Channel_ChannelLimitations');
        $oChannelLimitations = &new MockMAX_Maintenance_Forecasting_Channel_ChannelLimitations($this);
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Maintenance_Forecasting_Channel_ChannelLimitations', $oChannelLimitations);
        
        Mock::generate('MAX_Dal_Forecasting');
        $oDal = &new MockMAX_Dal_Forecasting($this);
        $oServiceLocator->register('MAX_Dal_Forecasting', $oDal);
        
        Mock::generate('MAX_Maintenance_Forecasting_Channel_Limitations');
        $oLimitations = &new MockMAX_Maintenance_Forecasting_Channel_Limitations($this);
        $oServiceLocator->register('MAX_Maintenance_Forecasting_Channel_Limitations', $oLimitations);
        
        $limitations = array();
        $limitations[] = array('type' => 'boo', 'data' => 'sample', 'comparison' => 'and', 'logical' => 'and');
        $limitations[] = array('type' => 'foo', 'data' => 'sample', 'comparison' => 'or',  'logical' => 'or');
        $limitations[] = array('type' => 'foo', 'data' => 'sample', 'comparison' => 'or',  'logical' => 'or');
        $this->limitations = $limitations;
    }
    
    /**
     * Test building sql limitations
     */
    function testBuildSqlLimitations()
    {
        $oServiceLocator = &ServiceLocator::instance();
        
        $chL = new MAX_Maintenance_Forecasting_Channel_ChannelLimitations();
        $chL->oLimitations = $oServiceLocator->get('MAX_Maintenance_Forecasting_Channel_Limitations');
        $chL->oLimitations->setReturnValueAt(0, 'isSqlLimitationExists', true);
        $chL->oLimitations->setReturnValueAt(1, 'isSqlLimitationExists', true);
        $chL->oLimitations->setReturnValueAt(2, 'isSqlLimitationExists', false);
        
        $limitations = $this->limitations;
        $amountOfLimitations = count($limitations);
        
        $chL->buildSqlLimitations($limitations, $remove = true);
        $this->assertEqual(count($limitations), $amountOfLimitations-2);
        
        $sqlLimitations = $chL->getSqlLimitations();
        $this->assertEqual(count($sqlLimitations), 2);
    }
    
    /**
     * Test building php limitations
     */
    function testBuildPhpLimitations()
    {
        $oServiceLocator = &ServiceLocator::instance();
        
        $chL = new MAX_Maintenance_Forecasting_Channel_ChannelLimitations();
        $chL->oLimitations = $oServiceLocator->get('MAX_Maintenance_Forecasting_Channel_Limitations');
        $chL->oLimitations->setReturnValue('getPhpLimitation', 'boo');
        
        $chL->buildPhpLimitations($this->limitations);
        
        $phpLimitations = $chL->getPhpLimitations();
        $this->assertTrue(!empty($phpLimitations));
    }
    
    /**
     * Test building limitations
     */
    function testBuildLimitations()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $chL = new MAX_Maintenance_Forecasting_Channel_ChannelLimitations();
        
        // Test 1 - all limitations could be translated into SQL
        $chL->oDal = $oServiceLocator->get('MAX_Dal_Forecasting');
        $chL->oDal->setReturnValue('getChannelLimitations', $this->limitations);
        $chL->oLimitations = $oServiceLocator->get('MAX_Maintenance_Forecasting_Channel_Limitations');
        $chL->oLimitations->setReturnValue('isSqlLimitationExists', true); // all sql limitations exists
        
        $chL->buildLimitations(123);
        $sqlLimitations = $chL->getSqlLimitations();
        $this->assertTrue(!empty($sqlLimitations));
        
        $phpLimitations = $chL->getPhpLimitations();
        $this->assertTrue(empty($phpLimitations));
        
        // Test 2 - not all limitations could be translated into SQL, but only ANDs exists so
        // SQL limitations are still in use
        $limitations = $this->limitations;
        foreach ($limitations as $key => $limitation) {
            $limitations[$key]['logical'] = 'and';
        }
        $chL->oDal = $oServiceLocator->get('MAX_Dal_Forecasting');
        $chL->oDal->setReturnValue('getChannelLimitations', $limitations);
        
        $chL->oLimitations = $oServiceLocator->get('MAX_Maintenance_Forecasting_Channel_Limitations');
        
        $chL->oLimitations->setReturnValue('isSqlLimitationExists', false, array('boo'));
        $chL->oLimitations->setReturnValue('isSqlLimitationExists', true, array('foo'));
        $chL->oLimitations->setReturnValue('getPhpLimitation', 'boo');
        
        $chL->buildLimitations(123);
        $sqlLimitations = $chL->getSqlLimitations();
        $this->assertTrue(!empty($sqlLimitations));
        
        $phpLimitations = $chL->getPhpLimitations();
        $this->assertTrue(!empty($phpLimitations));
    }
    


}

?>
