<?php
/**
 * Date range
 *
 * @since 0.3.19 - Feb 13, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id:DateRange.core.test.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

require_once('Date.php');
require_once(MAX_PATH . '/lib/max/DateRange.php');

class DateRangeTest extends UnitTestCase
{
    function testTodayFactory()
    {
        $now = new Date('1987-04-12 15:10:11');
        $today_start = new Date('1987-04-12 00:00:00');
        $today_end = new Date('1987-04-13 00:00:00');

        $range = DateRange::newToday($now);

        $this->assertEqual($range->getStartDate(), $today_start);
        $this->assertEqual($range->getEndDate(), $today_end, '"Today" ends at midnight tonight, which is the instant tomorrow begins. %s');
    }

    function testUseYesterday()
    {
        $now = new Date('2000-01-01 00:20:40');
        $yesterday_start = new Date('1999-12-31 00:00:00');
        $yesterday_end = new Date('2000-01-01 00:00:00');

        $range = new DateRange($now);
        $range->useYesterday();

        $this->assertEqual($range->getStartDate(), $yesterday_start);
        $this->assertEqual($range->getEndDate(), $yesterday_end);
    }

    function testUseLastWeek()
    {
        $now = new Date('2000-01-01 00:20:40');
        $week_start = new Date('1999-12-19 00:00:00');
        $week_end = new Date('1999-12-26 00:00:00');

        $range = new DateRange($now);
        $range->useLastWeek();

        $this->assertEqual($range->getStartDate(), $week_start);
        $this->assertEqual($range->getEndDate(), $week_end, '"Last week" ends on midnight spanning Saturday/Sunday. %s');
    }

    function testUseLast7Days()
    {
        $now = new Date('2000-01-19 15:20:40');
        $week_start = new Date('2000-01-12 00:00:00');
        $week_end = new Date('2000-01-19 00:00:00');

        $range = new DateRange($now);
        $range->useLast7Days();

        $this->assertEqual($range->getStartDate(), $week_start);
        $this->assertEqual($range->getEndDate(), $week_end);
    }

    function testUseLast17Days()
    {
        $now = new Date('2000-01-19 15:20:40');
        $week_start = new Date('2000-01-02 00:00:00');
        $week_end = new Date('2000-01-19 00:00:00');

        $range = new DateRange($now);
        $range->useLastDays(17);

        $this->assertEqual($range->getStartDate(), $week_start);
        $this->assertEqual($range->getEndDate(), $week_end);
    }

    function testUseLastMonth()
    {
        $now = new Date('2000-01-19 15:20:40');
        $month_start = new Date('1999-12-01 00:00:00');
        $month_end = new Date('2000-01-01 00:00:00');

        $range = new DateRange($now);
        $range->useLastMonth();

        $this->assertEqual($range->getStartDate(), $month_start);
        $this->assertEqual($range->getEndDate(), $month_end);
    }

    function testUseThisMonth()
    {
        $now = new Date('2000-01-19 15:20:40');
        $month_start = new Date('2000-01-01 00:00:00');
        $month_end = new Date('2000-02-01 00:00:00');

        $range = new DateRange($now);
        $range->useThisMonth();

        $this->assertEqual($range->getStartDate(), $month_start);
        $this->assertEqual($range->getEndDate(), $month_end);
    }

    function testUseNextMonth()
    {
        $now = new Date('2000-01-19 15:20:40');
        $month_start = new Date('2000-02-01 00:00:00');
        $month_end = new Date('2000-03-01 00:00:00');

        $range = new DateRange($now);
        $range->useNextMonth();

        $this->assertEqual($range->getStartDate(), $month_start);
        $this->assertEqual($range->getEndDate(), $month_end);
    }

    function testGetStartSql()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getStartSql();
        $this->assertEqual($sql, "'2000-01-19 00:00:00'");
    }

    function testGetEndSql_Seconds()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getEndSqlForComparingTimestamps();
        $this->assertEqual($sql, "'2000-01-20 00:00:00'");
    }

    function testGetEndSql_Days()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $sql = $range->getEndSqlForComparingDays();
        $this->assertEqual($sql, "'2000-01-19'");
    }

    function testGetStartDateForDisplay()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getStartDateForDisplay();
        $this->assertEqual($display, '19/01/2000');
    }

    function testGetEndDateForDisplay()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getEndDateForDisplay();
        $this->assertEqual($display, '19/01/2000');
    }

    function testGetStartDateForFilename()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getStartDateForFilename();
        $this->assertEqual($display, '2000-Jan-19');
    }

    function testGetEndDateForFilename()
    {
        $now = new Date('2000-01-19 15:20:40');
        $range = new DateRange($now);
        $range->useToday();
        $display = $range->getEndDateForFilename();
        $this->assertEqual($display, '2000-Jan-19');
    }

    function testHumansSeeWhatTheyPutIn()
    {
        $range = new DateRange();
        $range->setDateRangeByNaturalHumanStrings('2001-01-19', '2001-01-20');
        $display = $range->getEndDateForDisplay();
        $this->assertEqual($display, '20/01/2001', 'The last day of the range the user asks for should be the same as the one displayed, even if internal representation is different. %s');
    }

    function testCountMondaysInJune1999()
    {
        $now = new Date('1999-06-16');
        $range = new DateRange($now);
        $range->useThisMonth();
        $mondays = $range->countDayOfWeekOccurances(1);
        $this->assertEqual($mondays, 4, 'There were four Mondays in June 1999. %s');
    }

    function testCountMondaysInJune1998()
    {
        $now = new Date('1998-06-16');
        $range = new DateRange($now);
        $range->useThisMonth();
        $mondays = $range->countDayOfWeekOccurances(1);
        $this->assertEqual($mondays, 5, 'There were five Mondays in June 1998. %s');
    }

    function testCountSundays_ZeroBased()
    {
        $now = new Date('1998-06-16');
        $range = new DateRange($now);
        $range->useThisMonth();
        $sundays = $range->countDayOfWeekOccurances(0, 0);
        $this->assertEqual($sundays, 4, 'There were four Sundays in June 1998. %s');
    }

    function testCountSundays_SevenBased()
    {
        $now = new Date('1998-06-16');
        $range = new DateRange($now);
        $range->useThisMonth();
        $sundays = $range->countDayOfWeekOccurances(7, 7);
        $this->assertEqual($sundays, 4, 'There were four Sundays in June 1998. %s');
    }

    function testCountSundays_LastDayOfRangeIsCounted()
    {
        $now = new Date('2004-02-02');
        $range = new DateRange($now);
        $range->useThisMonth();
        $sundays = $range->countDayOfWeekOccurances(0);
        $this->assertEqual($sundays, 5, 'February 2004 began on a Sunday and ended on a Sunday. %s');
    }

    function testCountSundays_OnlyDaysInRangeAreCounted()
    {
        $now = new Date('2005-02-02');
        $range = new DateRange($now);
        $range->useThisMonth();
        $sundays = $range->countDayOfWeekOccurances(2);
        $this->assertEqual($sundays, 4, 'February 2005 began on a Tuesday and ended on a Monday. %s');
    }
}
?>
