<?php
/**
 * Report interface tests for Max Media Manager
 *
 * @since 0.3.19 - Mar 13, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: Reports.admin.test.php 4452 2006-03-13 18:03:17Z roh@m3.net $
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';
require_once MAX_PATH . '/plugins/reports/lib.php';

class NullReport extends Plugins_Reports
{
    // Minimum possible implementation
    // (no implementation at all)
}

class PublisherOnlyReport extends Plugins_Reports
{
    function initInfo()
    {
        $this->_authorize = phpAds_Publisher;
    }
}

class AlwaysDisplayableNeverExecutableReport extends Plugins_Reports
{
    function isAllowedToDisplay()
    {
        return true;
    }

    function isAllowedToExecute()
    {
        return false;
    }
}

class LegacyPublisherOnlyReport extends Plugins_Reports
{
    function info()
    {
        $info = array(
            'plugin-name' => 'Old plugin',
            'plugin-authorize' => phpAds_Publisher,
        );
        return $info;
    }
}

class ReportTest extends UnitTestCase
{
    function testInstantiation()
    {
        $report = new NullReport();
        $this->assertNoErrors();
    }

    function testCannotExecuteByDefault()
    {
        $report = new NullReport();
        $this->assertFalse($report->isAllowedToExecute(), 'No report may be executed without permission being granted in some form');
    }

    function testCanExecuteWhenAuthorizeSet()
    {
        global $session;

        $session['usertype'] = phpAds_Publisher;
        $report = new PublisherOnlyReport();
        $this->assertTrue($report->isAllowedToExecute(), 'A publisher should be allowed to execute a publisher report');
    }

    function testCanDisplayWhenAuthorizeSet()
    {
        global $session;

        $session['usertype'] = phpAds_Publisher;
        $report = new PublisherOnlyReport();
        $this->assertTrue($report->isAllowedToDisplay(), 'A publisher should see a publisher report');
    }

    function testCannotExecuteWhenAuthorizeSetWrong()
    {
        global $session;

        $session['usertype'] = phpAds_Advertiser;
        $report = new PublisherOnlyReport();
        $this->assertFalse($report->isAllowedToExecute(), 'An advertiser should not be allowed to execute a publisher report');
    }

    function testCustomDisplayAuthorisationLogic()
    {
        $report = new AlwaysDisplayableNeverExecutableReport();
        $this->assertTrue($report->isAllowedToDisplay(), 'Custom report logic says to always allow displaying');
    }

    function testCustomExecuteAuthorisationLogic()
    {
        $report = new AlwaysDisplayableNeverExecutableReport();
        $this->assertFalse($report->isAllowedToExecute(), 'Custom report logic says to never allow execution');
    }

    function testLegacyStyleReportDisplaysAsRightUser()
    {
        global $session;

        $session['usertype'] = phpAds_Publisher;
        $report = new LegacyPublisherOnlyReport();
        $this->assertTrue($report->isAllowedToDisplay(), 'A publisher should see a publisher report, even if it is an old-style report');
    }
}
?>
